********************************************************************************
* ɽ ϵήϳ ׻¸
*
*      (Lorenz, 1963)
*
*  99/06/21  ⹰
*      99/06/23 ;  
********************************************************************************
      program lore1

      parameter(imax=3,nmax=5000)
      parameter(dt=0.01,istep=1)

      real x(imax),work(3*imax),xp(nmax,imax)
      character cdummy*60

      common /consts/sigma,r,f,g
      external fcn,odrkg
      data sigma,r,f,g/10.0, 28.0, 8.0, 3.0/
*-----+--1---------2---------3---------4---------5---------6---------7-+
*-- initial conditions
      t = 0.0
      x(1) = 0.0
      x(2) = 1.0
      x(3) = 0.0

*-- time integrations
      do  n=1,nmax
        call odrkdu(imax, odrkg, fcn, t, t+dt, istep, x, work)
        do i=1,imax
          xp(n,i) = x(i)
        end do
      end do
      
*-- graphic output
      open (1,file='expar1.data')
      read (1,*) itype, r0, theta0, z0, cdummy
      close(1)
      call graph(nmax, imax, xp, itype, r0, theta0, z0)

      end
*-------------------------------------------------------------------------------
      subroutine fcn(imax, t, x, dx)

      real x(imax),dx(imax)

      common /consts/sigma,r,f,g
*-----+--1---------2---------3---------4---------5---------6---------7-+
      dx(1) = - sigma*x(1) + sigma*x(2)
      dx(2) =       r*x(1) -       x(2) - x(1)*x(3)
      dx(3) =                 x(1)*x(2) -  f/g*x(3)

      return
      end
*-------------------------------------------------------------------------------
      subroutine graph(nmax, imax, xp, itype, r0, theta0, z0)

      parameter(pi=3.141593)
      parameter(xymin=-35.,xymax=35.,zmin=0.,zmax=60.)
      parameter(vxymi=-0.5,vxyma=0.5,vzmi=0.,vzma=6./7.)
      parameter(kmax=540.)
      parameter(eps=0.4,period=1.)

      real xp(nmax,imax)
      character ct*60,cttl(3)*7,csgi*1

      common /consts/sigma,r,f,g
*-----+--1---------2---------3---------4---------5---------6---------7-+
      call swiset('iwidth', 720)
      call swiset('iheight',520)
      call swlset('lwait',.false.)
      call swlset('lalt',.true.)
      call sglset('lcorner',.false.)
      call uzfact(1.2)

      call gropn(1)

      call slmgn(0., 0., 0., 0.1)
      write(cttl(1),'(A2,f4.1)') csgi(154)//'=', sigma
      write(cttl(2),'(A2,f4.1)') csgi(163)//'=', r
      write(cttl(3),'(A2,I1,A1,I1)') 'b=',int(f),'/',int(g)
      ct = cttl(1) // cttl(2) // cttl(3)
      call slsttl(ct,'t', 1.0, 1.0, 0.015, 1)

* setup for itype=3
      a = r0/(1. + eps)
      dt = period/kmax
      theta2 = pi

      do k=0,kmax
        if(itype .eq. 1) then                             ! zޤβž
          theta = (theta0 - 2.*k) *pi/180.
          xeye = r0 * cos( theta )
          yeye = r0 * sin( theta )
          zeye = z0
          xobj = (xymax+xymin)/2
          yobj = xobj
          zobj = ( zmax+ zmin)/2
        else if(itype .eq. 2) then                        ! ľư
          r = r0 - 0.5*(k-1)
          xeye = r * cos( theta0*pi/180. )
          yeye = r * sin( theta0*pi/180. )
          zeye = z0
          xobj = xeye - 100.*cos( theta0*pi/180. )
          yobj = yeye - 100.*sin( theta0*pi/180. )
          zobj = z0
        else                                              ! Kepler-like
          theta  = 0.
          do i=0,k
            r = a *(1. - eps**2) /(1. + eps*cos(theta2) )
            theta = theta 
     +            + 2.*pi*a**2 *sqrt(1.-eps**2) /(period*r**2) *dt
            theta2 = pi + theta
          end do
          theta1 = theta0*pi/180. + theta
          xeye = r*sin(-theta1)
          yeye = r*cos(-theta1)
          zeye = r*cos(-theta)*z0/r0
          xobj = 0.
          yobj = 0.
          zobj = 30.
        end if
 
        xeye = xeye *(vxyma-vxymi)/(xymax-xymin)
        yeye = yeye *(vxyma-vxymi)/(xymax-xymin)
        zeye = zeye *( vzma- vzmi)/( zmax- zmin)
        xobj = xobj *(vxyma-vxymi)/(xymax-xymin)
        yobj = yobj *(vxyma-vxymi)/(xymax-xymin)
        zobj = zobj *( vzma- vzmi)/( zmax- zmin)

        call grfrm
        call g3dini(xymin, xymax, xymin, xymax, zmin, zmax,
     +              vxymi, vxyma, vxymi, vxyma, vzmi, vzma,
     +              xeye, yeye, zeye, xobj, yobj, zobj)

        call scspli(41)
        call scplu(nmax, xp(1,1), xp(1,2), xp(1,3))
      end do

      call grcls

      return
      end
********************************************************************************






