*****************************************************************************
*     time-dependent solutions of Lorenz's equations    (cf. Lorenz, 1963)
*     time-integrations by Runge-Kutta-Gill
*
*     ҤʽͰ¸
*
*	1997/08/31    (  礦)
*       1999/06/17 ; 
******************************************************************************
      program lore3

      parameter(imax=3,nmax=10000,ngrmax=10000)
      parameter(istep=1)
      parameter(dt=0.002,error=1.e-3)

      real x(2*imax),xp(0:ngrmax,2*imax),work(3*2*imax)
      real rmserr(0:ngrmax),time(0:ngrmax)
      character cdummy*60

      external fcn, odrkg
      common /consts/sigma,r,f,g
*-----+--1---------2---------3---------4---------5---------6---------7-+
*-- experimental parameters and initial conditions
      open (51,file='expar2.data')
      read (51,*) sigma, r, f, g, (x(i),i=1,3), cdummy
      close(51)

*-- integration for nmax steps to get the attractor
      t = 0.0
      call odrkdu(imax, odrkg, fcn, t, nmax*dt, nmax, x, work)

*-- restart
      t = 0.0
      do i=1,imax
           x(imax+i) = x(i) + error
        xp(0,     i) = x(i)
        xp(0,imax+i) = x(imax+i)
      end do
      rmserr(0) = error
        time(0) = t

      do n=1,ngrmax
        call odrkdu(2*imax, odrkg, fcn, t, t+dt, istep, x, work)
        do i=1,2*imax
          xp(n,i) = x(i)
        end do
        rmserr(n) = (x(1)-x(4))**2 + (x(2)-x(5))**2 + (x(3)-x(6))**2
        rmserr(n) = sqrt( rmserr(n)/3. )
          time(n) = t
      end do

*-- graphic
      call graph1(ngrmax, imax, xp)                           ! 3D
      call graph2(ngrmax, imax, xp, rmserr, time, error)      ! 2D

      end
********************************************************************************
      subroutine fcn(jmax, t, x, dx)

      real x(jmax), dx(jmax)

      common /consts/sigma,r,f,g
*-- system ----
      dx(1) = - sigma*x(1) + sigma*x(2)
      dx(2) =       r*x(1) -       x(2) - x(1)*x(3)
      dx(3) =                 x(1)*x(2) -  f/g*x(3)
      dx(4) = - sigma*x(4) + sigma*x(5)
      dx(5) =       r*x(4) -       x(5) - x(4)*x(6)
      dx(6) =                 x(4)*x(5) -  f/g*x(6)

      return
      end
******************************************************************************
      subroutine graph1(ngrmax, imax, xp)

      parameter(pi=3.141593)
      parameter(xymin=-35.,xymax=35.,zmin=0.,zmax=60.)
      parameter(vxymi=-0.5,vxyma=0.5,vzmi=0.,vzma=6./7.)
      parameter(xobj=(vxyma+vxymi)/2,yobj=xobj,zobj=(vzma+vzmi)/2)

      real xp(0:ngrmax,2*imax)
*-----+--1---------2---------3---------4---------5---------6---------7-+
      call swiset('iwidth', 720)
      call swiset('iheight',520)
      call sglset('lcorner',.false.)

      call uzfact(1.2)

      call gropn(1)
      call grfrm

      rinit  =   2.0
      angini = -65.
      zinit  =   1.0
      xeye = rinit*cos(angini*pi/180.)
      yeye = rinit*sin(angini*pi/180.)
      zeye = zinit

      call g3dini(xymin, xymax, xymin, xymax, zmin, zmax,
     +            vxymi, vxyma, vxymi, vxyma, vzmi, vzma,
     +            xeye, yeye, zeye, xobj, yobj, zobj)

      do n=0,ngrmax
        call scspmi(21)
        call scpmu(1,xp(n,4),xp(n,5),xp(n,6))
        call scspmi(41)
        call scpmu(1,xp(n,1),xp(n,2),xp(n,3))
      end do

      return
      end
******************************************************************************
      subroutine graph2(ngrmax, imax, xp, rmserr, time, error)

      parameter(ymin=-20., ymax=-ymin)

      real xp(0:ngrmax,2*imax),rmserr(0:ngrmax),time(0:ngrmax)
      character csgi*1,ct*60,cttl(3)*7

      common /consts/sigma,r,f,g
*-----+--1---------2---------3---------4---------5---------6---------7-+
      call glrget('rundef', rundef)
      call sglset('lfull',.true.)
      call sglset('lclip',.true.)
      call uzfact(0.8)
      call uziset('inner', 1)

*-- variavle-X
      call grfrm
      call grsvpt(0.18, 0.93, 0.38, 0.66)
      call grswnd(time(0), time(ngrmax), rundef, rundef)
      call usspnt(ngrmax+1, rundef, xp(0,1))
      call usspnt(ngrmax+1, rundef, xp(0,4))
      call uspfit
      call grstrn(1)
      call grstrf
      call scspln(2, 1, 0.0)
      call scseye(0.35, 0.5, -2.1)
      call scsobj(0.35, 0.5,  0.0)
      call scsprj

      call uzlset('labelxb',.false.)
      call uxaxdv('b', 1.0,  5.0)
      call uzlset('labelxb',.true.)
      call uxaxdv('t', 1.0,  5.0)
      call usyaxs('l')
      call usyaxs('r')
      call uysttl('l', 'X(T)', 0.0)

      write(cttl(1),'(A2,f4.1)') csgi(154)//'=', sigma
      write(cttl(2),'(A2,f4.1)') csgi(163)//'=', r
      write(cttl(3),'(A2,I1,A1,I1)') 'b=',int(f),'/',int(g)
      ct = cttl(1) // cttl(2) // cttl(3)
      call uxsttl('t', ct, 0.8)

      call sgspli(23)
      call sgplu(ngrmax+1, time, xp(0,4))
      call sgspli(43)
      call sgplu(ngrmax+1, time, xp(0,1))

*-- rms error
      call grfig
      call grswnd(time(0), time(ngrmax), 0.1*error, 90.)
      call grsvpt(0.18, 0.93, 0.08, 0.36)
      call grstrn(2)
      call grstrf
      call scspln(2, 1, 0.0)
      call scseye(0.35, 0.5, -2.1)
      call scsobj(0.35, 0.5,  0.0)
      call scsprj
      
      call uxaxdv('b', 1.0,  5.0)
      call uxaxdv('t', 1.0,  5.0)
      call ulylog('l', 1, 9)
      call ulylog('r', 1, 9)
      call uxsttl('b', 'TIME', 0.0)
      call uysttl('l', 'RMS ERROR', 0.0)

      call sgspli(33)
      call sgplu(ngrmax+1, time, rmserr)

      call grcls

      return
      end
********************************************************************************


