*****************************************************************************
*     time-dependent solutions of Lorenz's equations    (cf. Lorenz, 1963)
*     time-integrations by Runge-Kutta-Gill
*
*     ĥץå
*
*	1997/08/31    (  礦)
*       1999/06/17 ; 
******************************************************************************
      program lore4

      parameter(imax=3,nmax=150000,mmax=10000)
      parameter(dt=0.001, istep=1)

      real x(imax),work(3*imax),zp(nmax/10)
      character cdummy*60,csgi*1,ct*60,cttl(3)*7

      external fcn, odrkg
      common /consts/sigma,r,f,g
*-----+--1---------2---------3---------4---------5---------6---------7-+
*-- experimental parameters
      open (51,file='expar2.data')
      read (51,*) sigma, r, f, g, (x(i),i=1,3), cdummy
      close(51)

*-- initial conditions
      t    = 0.0 
      z2 = -999.0
      z3 = -999.0

*-- integration for nmax steps to get the attractor
      do n=1,nmax
        call odrkdu(imax, odrkg, fcn, t, t+dt, istep, x, work)
      end do

      m = 0
      do n=1,nmax
        call odrkdu(imax, odrkg, fcn, t, t+dt, istep, x, work)
        z1 = z2
        z2 = z3
        z3= x(3)
        if(z1.lt.z2 .and. z2.gt.z3) then
          m = m+1
          zp(m) = z2
        end if
      end do

*-- graphic open
      call swiset('iwidth', 720)
      call swiset('iheight',520)
      call sglset('lclip',.true.)
      call sglset('lcorner',.false.)
      call gropn(1)

      call grfrm
      call grsvpt( 0.15,  0.90,  0.15,  0.90)
      call usspnt(m, zp, zp)
      call uspfit
      call grstrn(1)
      call grstrf
      
      call ussttl('Z_n"', ' ', 'Z_n+1"', ' ')
      call usdaxs

      write(cttl(1),'(A2,f4.1)') csgi(154)//'=', sigma
      write(cttl(2),'(A2,f4.1)') csgi(163)//'=', r
      write(cttl(3),'(A2,I1,A1,I1)') 'b=',int(f),'/',int(g)
      ct = cttl(1) // cttl(2) // cttl(3)
      call uxsttl('t', ct, 1.0)

*-- restart
      t = 0.0
      zpn1 = zp(m)

      m = 0
      call sgspmi(41)
      do nn=1,nmax
        do n=1,nmax
          call odrkdu(imax, odrkg, fcn, t, t+dt, istep, x, work)
          z1 = z2
          z2 = z3
          z3= x(3)
          if(z1.lt.z2 .and. z2.gt.z3) then
            m = m+1
            zpn = zpn1
            zpn1  = z2
            call sgpmu(1, zpn, zpn1)
            if(m .ge. mmax) go to 999
          end if
        end do
      end do
      
  999 call grcls

      end
********************************************************************************
      subroutine fcn(jmax, t, x, dx)

      real x(jmax), dx(jmax)

      common /consts/sigma,r,f,g
*-- system ----
      dx(1) = - sigma*x(1) + sigma*x(2)
      dx(2) =       r*x(1) -       x(2) - x(1)*x(3)
      dx(3) =                 x(1)*x(2) -  f/g*x(3)

      return
      end
********************************************************************************
