********************************************************************************
* ɽ ϵήϳ ׻¸
*
*     ϹήĴᡧ2
*
*   An f-plane Two-dimensional Shallow Water Model(dimensinal form)
*           Kasahara's energy conservation scheme (1977)
*           periodic boundary condition
*           4th-order Runge-Kutta method for time-integration
*           Ref: W.M.Washington and C.L.Parkinson (1986)
*
*  91/xx/yy  
*      99/07/22 ; 
********************************************************************************
      program adjust

      implicit double precision(a-h,o-q,s-z)
      include 'incpa.f'

      dimension hu(imax,jmax),hv(imax,jmax), h(imax,jmax)
      real ru(iimax,jimax),rv(iimax,jimax),rh(iimax,jimax)
      real rmiss,rlngth,rdr,rek,rep,rekin,repin
      character cdummy*60

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      call gllset('lmiss', .true.)
      call glrget('rmiss',  rmiss)

      open(50,file='expar1.data')
      read(50,*) dlat, ijk, width, cdummy
      close(50)

      call const(rmiss, dlat, rlngth, rdr)
      call initia(ijk, width, hu, hv, h)
      call energy(hu, hv, h, rek, rep)
      call inner (hu, hv, h, ru, rv, rh)
      call enrgyi(ijk, ru, rv, rh, rekin, repin)
      rt = 0.0
      call grfini(rlngth, rdr, ru, rv, rh)

      open (60, file='adjst1.data', form='unformatted')
      write(60)   real(g), rlngth, rdr
      write(60)   rt, ru, rv, rh, rek, rep, rekin, repin
      call grfdrw(rt, ru, rv, rh)

      do n=1,nmax
        rt = dt*n/3600.            ! rt ñ̤ϻ
        call intgr (hu, hv, h)
        call energy(hu, hv, h, rek, rep)
        call inner (hu, hv, h, ru, rv, rh)
        call enrgyi(ijk, ru, rv, rh, rekin, repin)
        write(60)   rt, ru, rv, rh, rek, rep, rekin, repin
        call grfdrw(rt, ru, rv, rh)
      end do

      close(60)
      call grfcls

      stop
      end
********************************************************************************
      subroutine const(rmiss, dlat, rlngth, rdr)

      implicit double precision(a-h,o-z)
      include 'incpa.f'
      real rmiss,rlngth,rdr

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
*- constants
      pi     = 4.d0*atan(1.d0)
      omega  = 7.292d-5
      f      = 2.d0*omega*sin(dlat*pi/180.d0)
      g      = 9.81d0
      dlngth = 1.d7
      h0     = 1.d3

      if(f .ne. 0.0) then
        rdr = sqrt(g*h0)/f
      else
        rdr = rmiss
      end if

      write(*,*) rdr/1000.

*- domain size with dimension(mks)
      rlngth = dlngth
      dlngth = dlngth*3.d0             ! ΰޤ
      xlngth = dlngth
      ylngth = dlngth

*- grid size
      dx = xlngth/(imax-1)
      dy = ylngth/(jmax-1)
      dxinv = 1.d0/dx
      dyinv = 1.d0/dy

      return
      end
********************************************************************************
      subroutine initia(ijk, width, hu, hv, h)

      implicit double precision(a-h,o-z)
      include 'incpa.f'

      dimension hu(imax,jmax),hv(imax,jmax),h(imax,jmax)

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      xc = dx*(imax-1)*0.5d0
      yc = dy*(jmax-1)*0.5d0
      h1 = 1.d0
      v0 = 5.d-2

* ߷ʬ
      if(ijk .eq. 1) then
        do j=1,jmax
          do i=1,imax
            hu(i,j) = 0.d0
            hv(i,j) = 0.d0
            gauss   = - 0.5d0*(  (dx*(i-1)-xc)**2/width**2
     +                         + (dy*(j-1)-yc)**2/width**2 )
            h(i,j)  = h0 + h1*exp( gauss )
          end do
        end do
* Ӿʬ
      else if(ijk .eq. 2) then
        do j=1,jmax
          do i=1,imax
            hu(i,j) = 0.d0
            hv(i,j) = 0.d0
            gauss   = - 0.5d0*(  (dx*(i-1)-xc)**2/width**2 )
            h(i,j)  = h0 + h1*exp( gauss )
          end do
        end do
* ߷ή®ʬ
      else if(ijk .eq. 3) then
        do j=1,jmax
          do i=1,imax
            hu(i,j) = 0.d0
             h(i,j) = h0
            gauss   = - 0.5d0*(  (dx*(i-1)-xc)**2/width**2
     +                         + (dy*(j-1)-yc)**2/width**2 )
            hv(i,j)  = h0*v0*exp( gauss )
          end do
        end do
* Ӿή®ʬ
      else if(ijk .eq. 4) then
        do j=1,jmax
          do i=1,imax
            hu(i,j) = 0.d0
             h(i,j) = h0
            gauss   = - 0.5d0*(  (dx*(i-1)-xc)**2/width**2 )
            hv(i,j)  = h0*v0*exp( gauss )
          end do
        end do
      end if

      return
      end
********************************************************************************
      subroutine intgr(hu, hv, h)

      implicit double precision(a-h,o-z)
      include 'incpa.f'

      dimension hu (imax,jmax),hv (imax,jmax),h (imax,jmax)
      dimension hu1(imax,jmax),hv1(imax,jmax),h1(imax,jmax)
      dimension hu2(imax,jmax),hv2(imax,jmax),h2(imax,jmax)
      dimension hu3(imax,jmax),hv3(imax,jmax),h3(imax,jmax)
      dimension hu4(imax,jmax),hv4(imax,jmax),h4(imax,jmax)
      dimension huw(imax,jmax),hvw(imax,jmax),hw(imax,jmax)

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      do n=1,istep
        call delta(hu, hv, h, hu1, hv1, h1)
        do j=1,jmax
          do i=1,imax
            hu1(i,j) = dt*hu1(i,j)
            hv1(i,j) = dt*hv1(i,j)
             h1(i,j) = dt* h1(i,j)
            huw(i,j) = hu(i,j) + 0.5d0*hu1(i,j)
            hvw(i,j) = hv(i,j) + 0.5d0*hv1(i,j)
             hw(i,j) =  h(i,j) + 0.5d0* h1(i,j)
          end do
        end do

        call delta(huw, hvw, hw, hu2, hv2, h2)
        do j=1,jmax
          do i=1,imax
            hu2(i,j) = dt*hu2(i,j)
            hv2(i,j) = dt*hv2(i,j)
             h2(i,j) = dt* h2(i,j)
            huw(i,j) = hu(i,j) + 0.5d0*hu2(i,j)
            hvw(i,j) = hv(i,j) + 0.5d0*hv2(i,j)
             hw(i,j) =  h(i,j) + 0.5d0* h2(i,j)
          end do
        end do

        call delta(huw, hvw, hw, hu3, hv3, h3)
        do j=1,jmax
          do i=1,imax
            hu3(i,j) = dt*hu3(i,j)
            hv3(i,j) = dt*hv3(i,j)
             h3(i,j) = dt* h3(i,j)
            huw(i,j) = hu(i,j) + hu3(i,j)
            hvw(i,j) = hv(i,j) + hv3(i,j)
             hw(i,j) =  h(i,j) +  h3(i,j)
          end do
        end do

        call delta(huw, hvw, hw, hu4, hv4, h4)
        do j=1,jmax
          do i=1,imax
            hu4(i,j) = dt*hu4(i,j)
            hv4(i,j) = dt*hv4(i,j)
             h4(i,j) = dt* h4(i,j)
          end do
        end do

        do j=1,jmax
          do i=1,imax
            hu(i,j) = hu(i,j) 
     +            + (hu1(i,j)+2.d0*(hu2(i,j)+hu3(i,j))+hu4(i,j))/6.d0
            hv(i,j) = hv(i,j) 
     +            + (hv1(i,j)+2.d0*(hv2(i,j)+hv3(i,j))+hv4(i,j))/6.d0
            h(i,j) =  h(i,j) 
     +            + ( h1(i,j)+2.d0*( h2(i,j)+ h3(i,j))+ h4(i,j))/6.d0
          end do
        end do
      end do

      return
      end
********************************************************************************
      subroutine delta(hu, hv, h, dhu, dhv, dh)

      implicit double precision(a-h,o-z)
      include 'incpa.f'

      dimension  hu(imax,jmax), hv(imax,jmax), h(imax,jmax),
     +          dhu(imax,jmax),dhv(imax,jmax),dh(imax,jmax),
     +            u(imax,jmax),  v(imax,jmax)

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      do j=1,jmax
        do i=1,imax
          u(i,j) = hu(i,j)/h(i,j)
          v(i,j) = hv(i,j)/h(i,j)
        end do
      end do

      do j=2,jmax-1
        do i=2,imax-1
          huip = 0.5d0*( hu(i+1,j) + hu(i,j) )
           uip = 0.5d0*(  u(i+1,j) +  u(i,j) )
           vip = 0.5d0*(  v(i+1,j) +  v(i,j) )
           hip = 0.5d0*(  h(i+1,j) +  h(i,j) )
          huim = 0.5d0*( hu(i-1,j) + hu(i,j) )
           uim = 0.5d0*(  u(i-1,j) +  u(i,j) )
           vim = 0.5d0*(  v(i-1,j) +  v(i,j) )
           him = 0.5d0*(  h(i-1,j) +  h(i,j) )

          hujp = 0.5d0*( hu(i,j+1) + hu(i,j) )
          hvjp = 0.5d0*( hv(i,j+1) + hv(i,j) )
           vjp = 0.5d0*(  v(i,j+1) +  v(i,j) )
           hjp = 0.5d0*(  h(i,j+1) +  h(i,j) )
          hujm = 0.5d0*( hu(i,j-1) + hu(i,j) )
          hvjm = 0.5d0*( hv(i,j-1) + hv(i,j) )
           vjm = 0.5d0*(  v(i,j-1) +  v(i,j) )
           hjm = 0.5d0*(  h(i,j-1) +  h(i,j) )

          dhu(i,j) = - dxinv*( huip*uip - huim*uim )
     +               - dyinv*( hujp*vjp - hujm*vjm )
     +               + f*hv(i,j) 
     +               - g* h(i,j)*dxinv*( hip - him )

          dhv(i,j) = - dxinv*( huip*vip - huim*vim )
     +               - dyinv*( hvjp*vjp - hvjm*vjm )
     +               - f*hu(i,j) 
     +               - g* h(i,j)*dyinv*( hjp - hjm )

          dh (i,j) = - dxinv*( huip - huim )
     +               - dyinv*( hvjp - hvjm )
        end do
      end do

      huip = 0.5d0*( hu(     2,1) + hu(1,1) )
       uip = 0.5d0*(  u(     2,1) +  u(1,1) )
       vip = 0.5d0*(  v(     2,1) +  v(1,1) )
       hip = 0.5d0*(  h(     2,1) +  h(1,1) )
      huim = 0.5d0*( hu(imax-1,1) + hu(1,1) )
       uim = 0.5d0*(  u(imax-1,1) +  u(1,1) )
       vim = 0.5d0*(  v(imax-1,1) +  v(1,1) )
       him = 0.5d0*(  h(imax-1,1) +  h(1,1) )

      hujp = 0.5d0*( hu(1,     2) + hu(1,1) )
      hvjp = 0.5d0*( hv(1,     2) + hv(1,1) )
       vjp = 0.5d0*(  v(1,     2) +  v(1,1) )
       hjp = 0.5d0*(  h(1,     2) +  h(1,1) )
      hujm = 0.5d0*( hu(1,jmax-1) + hu(1,1) )
      hvjm = 0.5d0*( hv(1,jmax-1) + hv(1,1) )
       vjm = 0.5d0*(  v(1,jmax-1) +  v(1,1) )
       hjm = 0.5d0*(  h(1,jmax-1) +  h(1,1) )

      dhu(1,1) = - dxinv*( huip*uip - huim*uim )
     +           - dyinv*( hujp*vjp - hujm*vjm )
     +           + f*hv(1,1) 
     +           - g* h(1,1)*dxinv*( hip - him )

      dhv(1,1) = - dxinv*( huip*vip - huim*vim )
     +           - dyinv*( hvjp*vjp - hvjm*vjm )
     +           - f*hu(1,1) 
     +           - g* h(1,1)*dyinv*( hjp - hjm )

      dh (1,1) = - dxinv*( huip - huim )
     +           - dyinv*( hvjp - hvjm )

      do i=2,imax-1
        huip = 0.5d0*( hu(i+1,1) + hu(i,1) )
         uip = 0.5d0*(  u(i+1,1) +  u(i,1) )
         vip = 0.5d0*(  v(i+1,1) +  v(i,1) )
         hip = 0.5d0*(  h(i+1,1) +  h(i,1) )
        huim = 0.5d0*( hu(i-1,1) + hu(i,1) )
         uim = 0.5d0*(  u(i-1,1) +  u(i,1) )
         vim = 0.5d0*(  v(i-1,1) +  v(i,1) )
         him = 0.5d0*(  h(i-1,1) +  h(i,1) )

        hujp = 0.5d0*( hu(i,     2) + hu(i,1) )
        hvjp = 0.5d0*( hv(i,     2) + hv(i,1) )
         vjp = 0.5d0*(  v(i,     2) +  v(i,1) )
         hjp = 0.5d0*(  h(i,     2) +  h(i,1) )
        hujm = 0.5d0*( hu(i,jmax-1) + hu(i,1) )
        hvjm = 0.5d0*( hv(i,jmax-1) + hv(i,1) )
         vjm = 0.5d0*(  v(i,jmax-1) +  v(i,1) )
         hjm = 0.5d0*(  h(i,jmax-1) +  h(i,1) )

        dhu(i,1) = - dxinv*( huip*uip - huim*uim )
     +             - dyinv*( hujp*vjp - hujm*vjm )
     +             + f*hv(i,1) 
     +             - g* h(i,1)*dxinv*( hip - him )

        dhv(i,1) = - dxinv*( huip*vip - huim*vim )
     +             - dyinv*( hvjp*vjp - hvjm*vjm )
     +             - f*hu(i,1) 
     +             - g* h(i,1)*dyinv*( hjp - hjm )

        dh (i,1) = - dxinv*( huip - huim )
     +             - dyinv*( hvjp - hvjm )
      end do

      do j=2,jmax-1
        huip = 0.5d0*( hu(     2,j) + hu(1,j) )
         uip = 0.5d0*(  u(     2,j) +  u(1,j) )
         vip = 0.5d0*(  v(     2,j) +  v(1,j) )
         hip = 0.5d0*(  h(     2,j) +  h(1,j) )
        huim = 0.5d0*( hu(imax-1,j) + hu(1,j) )
         uim = 0.5d0*(  u(imax-1,j) +  u(1,j) )
         vim = 0.5d0*(  v(imax-1,j) +  v(1,j) )
         him = 0.5d0*(  h(imax-1,j) +  h(1,j) )

        hujp = 0.5d0*( hu(1,j+1) + hu(1,j) )
        hvjp = 0.5d0*( hv(1,j+1) + hv(1,j) )
         vjp = 0.5d0*(  v(1,j+1) +  v(1,j) )
         hjp = 0.5d0*(  h(1,j+1) +  h(1,j) )
        hujm = 0.5d0*( hu(1,j-1) + hu(1,j) )
        hvjm = 0.5d0*( hv(1,j-1) + hv(1,j) )
         vjm = 0.5d0*(  v(1,j-1) +  v(1,j) )
         hjm = 0.5d0*(  h(1,j-1) +  h(1,j) )

        dhu(1,j) = - dxinv*( huip*uip - huim*uim )
     +             - dyinv*( hujp*vjp - hujm*vjm )
     +             + f*hv(1,j) 
     +             - g* h(1,j)*dxinv*( hip - him )

        dhv(1,j) = - dxinv*( huip*vip - huim*vim )
     +             - dyinv*( hvjp*vjp - hvjm*vjm )
     +             - f*hu(1,j) 
     +             - g* h(1,j)*dyinv*( hjp - hjm )

        dh (1,j) = - dxinv*( huip - huim )
     +             - dyinv*( hvjp - hvjm )
      end do

      do i=1,imax-1
        dhu(i,jmax) = dhu(i,1)
        dhv(i,jmax) = dhv(i,1)
        dh (i,jmax) = dh (i,1)
      end do

      do j=1,jmax-1
        dhu(imax,j) = dhu(1,j)
        dhv(imax,j) = dhv(1,j)
        dh (imax,j) = dh (1,j)
      end do

      dhu(imax,jmax) = dhu(1,1)
      dhv(imax,jmax) = dhv(1,1)
      dh (imax,jmax) = dh (1,1)

      return
      end
********************************************************************************
      subroutine inner(hu, hv, h, ru, rv, rh)

      implicit double precision(a-h,o-q,s-z)
      include 'incpa.f'

      dimension hu(imax,jmax),hv(imax,jmax), h(imax,jmax)
      real  ru(iimax,jimax),rv(iimax,jimax),rh(iimax,jimax), rmiss
      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      call gllset('lmiss', .true.)
      call glrget('rmiss',  rmiss)
      
      do j=1,jimax
        do i=1,iimax
          ii = i + iimax - 1
          ji = j + jimax - 1
          if(h(ii,jj) .eq. 0.d0) then
            ru(i,j) = rmiss
            rv(i,j) = rmiss
          else
            ru(i,j) = hu(ii,ji)/h(ii,ji)
            rv(i,j) = hv(ii,ji)/h(ii,ji)
          end if
          rh(i,j) = h(ii,ji) - h0
        end do
      end do
      
      return
      end
********************************************************************************
      subroutine energy(hu, hv, h, rek, rep)

      implicit double precision(a-h,o-z)
      include 'incpa.f'

      dimension hu(imax,jmax),hv(imax,jmax),h(imax,jmax)
      real rek, rep

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      rek = 0.0
      rep = 0.0
      do j=1,jmax-1
        do i=1,imax-1
          u = hu(i,j)/h(i,j)
          v = hv(i,j)/h(i,j)
          hpert = h(i,j) - h0
          rek = rek + 0.5d0*h(i,j)*(u**2 + v**2)
          rep = rep + 0.5d0*g*hpert**2
        end do
      end do

      return
      end
********************************************************************************
      subroutine enrgyi(ijk, ru, rv, rh, rekin, repin)

      implicit double precision(a-h,o-z)
      include 'incpa.f'

      real ru(iimax,jimax),rv(iimax,jimax),rh(iimax,jimax)
      real rekin, repin

      common /const1/ f,g,h0,dx,dy,dxinv,dyinv
*-----+--1---------2---------3---------4---------5---------6---------7-+
      rekin = 0.0
      repin = 0.0

      do j=1,jimax-1
        do i=1,iimax-1
          htot = rh(i,j) + h0
          rekin = rekin + 0.5d0*htot*(ru(i,j)**2 + rv(i,j)**2)
          repin = repin + 0.5d0*g*rh(i,j)**2
        end do
      end do

      if(mod(ijk,2) .eq. 0) then
        rekin = rekin*3.0                ! Ӿξξ٤ΰޤ
        repin = repin*3.0
      end if

      return
      end
********************************************************************************
