***********************************************************************
*     Dispersion Relation of Hough Functions for Given Zonal Wave Number
*     Equiv. depth - Frequency cross section
*     Solve Eigen Function Problem for Equivalent Deapth.
*     log-log plot.
*     Accuracy is tested by compareing E and E2 (order OACC)
*       last revised   Mar 1993   Horinouchi,T. 
*       (addapted to dcl-5, Apr 1997, Horiouchi)
***********************************************************************
c     Parameters :
c       JG: number of division for SGM
c       MM: maximum total wave number for calculation for E
c       MM2(=.7*MM): same as MM but for E2
c       M : zonal wave number
c       NN: dimension of the matrix of coefficient of E (NN2 for E2)
c       B1,B2: maximum,minimum of 'beki' of SGM
c       OACC: order of the accuracy of chosen eigen values
c       LMAX: number for plot each mode
c     Variables :
c       SGM: normalized frequency s.t. SGM=1 for semidiurnal mode
c       E: eigen values which correspond -1*(nondim. equivalent depth).
c       E2: eigen values which correspond -1*(nondim. equivalent depth).
c    +      associated function for geopotential field.
c       NS: symmetric(=0) or antisymmetric(=1)
*======================================================================
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      REAL*8 OACC,RMISS
      PARAMETER(B1=-2D0,B2=1D0,OACC=1D-3,LMAX=5,LMX=3*LMAX)
      PARAMETER(MM=90,NN1=MM/2+1,JG=200,UMISS=0.,RMISS=0D0)
      DIMENSION D(NN1),SD(NN1),E(NN1),E2(NN1),A(LMX)
      DIMENSION UX(LMX,JG),UYP(LMX,JG,0:1),UYN(LMX,JG,0:1)
      MM2=MM*0.7
      WRITE(6,*) 'Zonal Wave Number (0,1,2,..) ;'
      READ(5,*) M
      WRITE(6,*) 'Eastward(=1) or Westward(=-1);'
      READ(5,*) ISGN
      CALL SGPWSN
      READ(5,*) IWS

      CALL GLPSET('LMISS',.TRUE.)
      CALL GLPSET('RMISS',UMISS)

      DO 10 J=1,JG
      DO 20 NS=0,1
        SGM=ISGN*10D0**(1D0*(B2-B1)*J/JG+B1)
        CALL JUDGE(M,SGM,ICON)
        IF(ICON.EQ.0) THEN
        NN=(MM+NS-M)/2-NS+1
        CALL MATRIX(M,SGM,NS,NN,D,SD)
        CALL DTRQL(D,SD,NN,E,ND,ICON)
        CALL BUBBLE(ND,E)
        NN2=(MM2+NS-M)/2-NS+1
        CALL MATRIX(M,SGM,NS,NN2,D,SD)
        CALL DTRQL(D,SD,NN2,E2,ND2,ICON)
        CALL BUBBLE(ND2,E2)
        CALL CHOOSE(ND,E,ND2,E2,A,LMAX,OACC,SGM,M,NS,RMISS)
        CALL CHANGE(LMX,A,F)
        CALL SETUXY(ABS(SGM),LMX,A,UX(1,J),UYP(1,J,NS),UYN(1,J,NS),F)
        ENDIF
   20 CONTINUE
      IF(MOD(J,20).EQ.0) WRITE(6,*) 'FINISHED ',J,'/',JG
   10 CONTINUE
      CALL GROPEN(IWS)
      CALL SLMGN(0.0,0.0,0.0,0.15)
      CALL SLDIV('Y',2,1)
      CALL TITLE(M,ISGN)
      CALL SLRAT(0.7,1.0)
      CALL GRAPH(LMX,JG,UX,UYP,ISGN)
      CALL GGXTTL(1)
      CALL GRAPH(LMX,JG,UX,UYN,ISGN)
      CALL GGXTTL(-1)
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE GROPEN(IWS)
 
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UDPSET('LMSG',.FALSE.)
 
      RETURN
      END
************************************************************************
C     dimensionalize equivalent depth

      SUBROUTINE CHANGE(NN,E,F)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN)
 
      A=6.37D6
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      G=9.8D0
      F=4*A*A*OME*OME/G*1D-3
 
      DO 20 ND=1,NN
      E(ND)=-F*E(ND)
   20 CONTINUE
 
      RETURN
      END
************************************************************************
c     set the coeficint matrix

      SUBROUTINE MATRIX(M,SGM,NS,NN,D,SD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION D(NN),SD(NN)
 
      R(N)=SQRT(1D0*(N*N-M*M)/(4*N*N-1))
      S(N)=M/SGM+N*(N+1)
      T1(N)=R(N)*R(N)*(N-1)*(N-1)/(S(N-1)*N*N)
      T2(N)=S(N)*SGM*SGM/(N*N*(N+1)*(N+1))
      T3(N)=R(N+1)*R(N+1)*(N+2)*(N+2)/(S(N+1)*(N+1)*(N+1))
 
      DO 10 ND=2,NN
        N=M+NS+ND*2-2
        D(ND)=T1(N)-T2(N)+T3(N)
        N=N-1
        SD(ND)=R(N)*R(N+1)/S(N)
   10 CONTINUE
 
      IF(NS.EQ.0 .AND. M.NE.0) THEN
        N=M 
        D(1)=-T2(N)+T3(N)
      ELSE IF(NS.EQ.1 .AND. M.NE.0) THEN
        N=M+1
        D(1)=T1(N)-T2(N)+T3(N)
      ELSE IF(NS.EQ.0 .AND. M.EQ.0) THEN
        D(1)=0D0
        SD(2)=0D0
      ELSE IF(NS.EQ.1 .AND. M.EQ.0) THEN
        N=M+1
        D(1)=-T2(N)+T3(N)
      END IF
      
      RETURN
      END
************************************************************************
      SUBROUTINE TITLE(M,ISGN)
 
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER CS*3,CT*60
 
      CS='s=# '
      WRITE(CS(3:3),'(I1)') M
 
      CALL SGPSET('INDEX',2)
      IF(ISGN.EQ.1) THEN
      CALL SLSTTL('Eastward Moving Hough Modes '//CS,'T',0.0,0.0,0.03,1)
      ELSE
      CALL SLSTTL('Westward Moving Hough Modes '//CS,'T',0.0,0.0,0.03,1)
      ENDIF
**      CT='Relation Between Equivalent Depth and Frequency'
**      CALL SLSTTL(CT,'T',0.0,-1.0,0.025,2)
 
      RETURN
      END
************************************************************************
      SUBROUTINE JUDGE(M,SGM,ICON)

      IMPLICIT REAL*8(A-H,O-Z)
      ICON=0
      IF (SGM.EQ.0) THEN
        WRITE(6,*) 'SGM=0'
        ICON=1000
        STOP
      ENDIF
      IF (M/SGM.GE.0) RETURN
      S=(-1D0+SQRT(1D0-4*M/SGM))/2
      SS=INT(S)
      IF (S.EQ.SS .AND. SS.GE.M) THEN
        WRITE(6,*) 'THERE EXISTS INTEGER N S.T. M/SGM=-N*(N+1)'
        ICON=2000
      ENDIF
      RETURN
      END
********************************************************************** *
c     sorting by 'awadachi' method

      SUBROUTINE BUBBLE(NN,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN)
 
   30 CONTINUE
      INDEX=0
      DO 20 N=1,NN-1
      IF(E(N).GT.E(N+1)) THEN
      ED=E(N)
      E(N)=E(N+1)
      E(N+1)=ED
      INDEX=1      
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30
 
      RETURN
      END
************************************************************************
C     < preparation for graphics > 

      SUBROUTINE SETUXY(X,KM,E,UX,UYP,UYN,F)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(KM),UX(KM),UYP(KM),UYN(KM)
     
      DO 10 K=1,KM
      ED=E(K)
      UX(K)=X
      IF(ED.LT.0D0) THEN
        UYN(K)=SQRT(-ED/F)
      ELSE IF(ED.GT.0D0) THEN
        UYP(K)=SQRT(ED/F)
      END IF
   10 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(LMX,JG,UX,UY,ISGN,F)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION UX(LMX,JG),UY(LMX,JG,0:1)
      CHARACTER CSGI*1,CS*6,CE*16
 
      UYMAX=UX(1,JG)
      UYMIN=UX(1,1)
      UXMIN=10E-3
      UXMAX=10E0
      
      CALL GRFRM
      CALL SGPSET('LCLIP',.TRUE.)
      CALL GRSVPT(0.2,0.6,0.2,0.9)
      CALL GRSWND(UXMIN,UXMAX,UYMIN,UYMAX)
      CALL GRSTRN (4)
      CALL GRSTRF
      CALL ULXLOG('T',1,9)
      CALL ULXLOG('B',1,9)
      CALL ULYLOG('L',1,9)
      CALL ULYLOG('R',1,9)
      CALL SGSPMS(0.004)
      CALL SGSPMT(2)
      CALL SGPMU(LMX*JG,UY(1,1,0),UX)
      CALL SGSPMS(0.003)
      CALL SGSPMT(5)
      CALL SGPMU(LMX*JG,UY(1,1,1),UX)
      RETURN
 
      ENTRY GGXTTL(I)
      CS='(*2O)'
      CS(2:2)=CSGI(5)
      CS(4:4)=CSGI(151)
      CALL UYSTTL('L','Frequency'//CS,0.0)
      IF(I.EQ.-1) THEN
        CE='#(gh)/2#a'
        CE(1:1)=CSGI(210)
        CE(8:8)=CSGI(151)
        CALL UXSTTL('B',CE,0.0)
        CALL UXMTTL('T','negative h',0.0)
      ELSE
        CE='#(-gh)/2#a'
        CE(1:1)=CSGI(210)
        CE(9:9)=CSGI(151)
        CALL UXSTTL('B',CE,0.0)
        CALL UXMTTL('T','positive h',0.0)
      ENDIF
      RETURN
      END
*******************************************************************
C     < chose reliable eigen values order OACC>
 
      SUBROUTINE CHOOSE(ND1,E1,ND2,E2,A,LMAX,OACC,SGM,M,NS,RMISS)

C     Ei(i=1,2): eigen values (-1/eps)
C     OACC: order of the accuracy of chosen eigen values
C     NC: number of chosen values
C     NS: symmetric(=0) or antisymmetric(=1)
C     SGM: normalized frequency s.t. SGM=1 for semidiurnal mode
C     S2GRVi(i=1,2): (freq. of pure gravity wave)**2
C     M: zonal wave number
     
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E1(ND1),E2(ND2),A(3*LMAX)

      IF(ND2.GT.ND1) THEN
        WRITE(6,*) 'ERROR IN SUBROUTINE CHOOSE : ND2 MUST BE .LE. ND1'
        STOP
      ENDIF
      DO 5 K=1,3*LMAX
        A(K)=RMISS
    5 CONTINUE

C     < count number of positive eps >

      DO 11 N=1,ND1
        IF(E1(N).LT.0) NP1=N
   11 CONTINUE
      DO 12 N=1,ND2
        IF(E2(N).LT.0) NP2=N
   12 CONTINUE

C     < set negative accurate eps >

      NAN=0
      DO 20 N=1,MIN(ND1-NP1,ND2-NP2)
        A1=ABS(E1(ND1-N+1))
        A2=ABS(E2(ND2-N+1))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAN=NAN+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAN=NAN+1
        ELSE
          NAN=NAN+1
        ENDIF
   20 CONTINUE
      IF(SGM.GE.0)THEN
        LL=0
      ELSE
        LLL=( -1+SQRT(1-4*M/SGM) )/2 -(M-1)
        LL=MAX( 0, (LLL+NS)/2 )
      ENDIF
      LMAXX=LMAX-LL
      DO 30 N=1,MIN(NAN,LMAXX)
        A(N)=E1(ND1-N+1)
   30 CONTINUE
      NC=NAN

C     < set positive accurate eps >

      IF(SGM.GT.0) THEN

      NAP=0
      DO 40 N=1,MIN(NP1,NP2)
        A1=ABS(E1(N))
        A2=ABS(E2(N))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAP=NAP+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAP=NAP+1
        ELSE
          NAP=NAP+1
        ENDIF
   40 CONTINUE
      DO 50 N=1,MIN(NAP,LMAX)
        A(N+LMAX)=E1(N)
   50 CONTINUE
      NC=NAN+NAP

      ELSE

C     :count the numbers of Rosby and Gravity wave and set them:
      SGMSQ=SGM*SGM
      NROS1=0
      DO 61 N=1,NP1
        S2GRV1=M*M*(-E1(N))
        IF(S2GRV1.GE.SGMSQ) NROS1=NROS1+1
   61 CONTINUE
      NROS2=0
      DO 62 N=1,NP2
        S2GRV2=M*M*(-E2(N))
        IF(S2GRV2.GE.SGMSQ) NROS2=NROS2+1
   62 CONTINUE
      NAR=0
      DO 70 N=1,MIN(NROS1,NROS2)
        A1=ABS(E1(NROS1-N+1))
        A2=ABS(E2(NROS2-N+1))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAR=NAR+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAR=NAR+1
        ELSE
          NAR=NAR+1
        ENDIF
   70 CONTINUE
      DO 80 N=1,MIN(NAR,LMAX)
        A(N+LMAX)=E1(NROS1-N+1)
   80 CONTINUE
      NAG=0
      DO 90 N=1,MIN(NP1-NROS1,NP2-NROS2)
        A1=ABS(E1(NROS1+N))
        A2=ABS(E2(NROS2+N))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAG=NAG+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAG=NAG+1
        ELSE
          NAG=NAG+1
        ENDIF
   90 CONTINUE
      DO 100 N=1,MIN(NAG,LMAX)
        A(N+2*LMAX)=E1(NROS1+N)
  100 CONTINUE
      NC=NAN+NAR+NAG

      ENDIF
      RETURN
      END
