* Compile: (dcl5) dclfrt bunsanHFP.f -L/home/horinout/lib -lssl
************************************************************************
*     Dispersion Relation of Hough Functions for Given Zonal Wave Number
*     Equiv. depth - Frequency cross section
*     Solve Eigen Function Problem for Frequency.
*     log-log plot.
*       revised   Mar 1993   Horinouchi T. 
*       revised   Oct 1995   Horinouchi T. 
*       (addapted to dcl-5, Apr 1997, Horiouchi)
************************************************************************
C     Parameters :
C       JG: number of division for SGM
C       M : zonal wave number
C       MM: maximum total wave number for calculation 
C       MM2(=.7*MM): same as MM but for E2
C       OACC: relative Order for ACCuracy
C       LMAX: number for plot each mode
C     Variables :
C       E: eigen values which correspond equivalent depth.
C    +     associated function for geopotential field.
C       SGM: normalized angular freq. s.t. SGM=1 for semidiurnal mode
C       NS: symmetric(=0) or antisymmetric(=1)
*====================================================================== 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      PARAMETER(MMI=90,MMX=150,KMM=3*(MMX/2+1),LM=KMM*(KMM+1)/2)
      PARAMETER(JG=60,UMISS=0.)
      DIMENSION A(LM),D(KMM),SD(KMM),E(KMM)
      PARAMETER(NEG=3,NWG=3,NRS=3)
      DIMENSION EEG(NEG),EWG(NWG),ERS(NRS)
      DIMENSION UX(0:JG),UYP(0:JG,NEG,0:1),UYN(0:JG,NWG+NRS,0:1)
      LOGICAL LFIRST
      DATA    LFIRST/.TRUE./

      RE=6.37D6
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      G=9.8D0
      F=4*RE*RE*OME*OME/G
      WRITE(6,*) 'Zonal Wave Number;'
      READ(5,*) M
      CALL SGPWSN
      READ(5,*) IWS

      MM=MIN(MMI+M,MMX)

      CALL GLPSET('LMISS',.TRUE.)
      CALL GLPSET('RMISS',UMISS)

      DO 10 J=0,JG
        BK1=-4D0
        BK2=2D0
        EPS=10D0**(-1D0*(BK1+(BK2-BK1)*J/JG))
        DO 20 IS=0,1
          CALL SETPAR(MM,M,IS,KM,NS,ID)
          CALL MATRIX(A,M,KM,NS,ID,EPS)
          CALL DTRID1(A,KM,D,SD,ICON)
          CALL DTRQL(D,SD,KM,E,KMD,ICON)
          CALL BUBBLE(KMD,E)
          CALL CHOSE(KMD,E,EPS,M,UMISS,NEG,NWG,NRS, EEG,EWG,ERS)
          CALL SETUXY(J,JG,NEG,NWG,NRS,EEG,EWG,ERS, EPS,F,
     +                UX(J),UYP(0,1,IS),UYN(0,1,IS))
   20   CONTINUE
        IF(MOD(J,5).EQ.0) WRITE(6,*) 'Finished ',J,'/',JG
   10 CONTINUE
      CALL GROPEN(IWS)
      CALL SLMGN(0.0,0.0,0.0,0.15)
      CALL SLDIV('Y',2,1)
      CALL TITLE(M)
      CALL SLRAT(0.7,1.0)
      CALL GRAPH(JG,NEG,UX,UYP)
      CALL GYTTL(1)
      CALL GRAPH(JG,NWG+NRS,UX,UYN)
      CALL GYTTL(-1)
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE GROPEN(IWS)
 
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
**      CALL SGPSET('LCLIP',.TRUE.)

      RETURN
      END
************************************************************************
      SUBROUTINE MATRIX(A,M,KM,NS,ID,EPS)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(KM*(KM+1)/2)
 
      R(N)=-1D0*M/(N*(N+1))
      S(N)=-SQRT(N*(N+1)/EPS)
      T(N)=SQRT(1D0*(N*N-1)*(N*N-M*M)/(4*N*N-1))/N
 
      DO 10 I=1,KM*(KM+1)/2
      A(I)=0
   10 CONTINUE
 
      N=NS
      L=1
      DO 20 K=1,KM
      JR=MOD(K-1-ID,3)
      IF(JR.NE.0) THEN
      A(L)=R(N)
      N=N+1
      END IF
      L=L+K+1
   20 CONTINUE
 
      N=NS+ID
      L=2
      DO 30 K=2,KM
      JS=MOD(K-2-ID,3)
      JT=MOD(K-ID,3)
      IF(JS.EQ.0) THEN
      A(L)=S(N)
      N=N+1
      ELSE IF(JT.EQ.0) THEN
      A(L)=T(N)
      N=N+1
      END IF
      L=L+K+1
   30 CONTINUE
 
      N=NS+2-ID
      L=4
      DO 40 K=3,KM
      JS=MOD(K-2-ID,3)
      IF(JS.EQ.0) THEN
      A(L)=T(N)
      N=N+2
      END IF
      L=L+K+1
   40 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE TITLE(M)
 
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER CS*4   !",CT*60
 
      CS='s=#'
      WRITE(CS(3:4),'(I2)') M
 
      CALL SGPSET('INDEX',2)
**      CALL SLSTTL('Hough Modes '//CS,'T',0.0,0.3,0.03,1)
      CALL SLSTTL('Hough Modes '//CS,'T',0.0,-0.2,0.03,1)
**      CT='Relation Between Positive Equivalent Depth and Frequency'
**      CALL SLSTTL(CT,'T',0.0,-1.0,0.025,2)
 
      RETURN
      END
************************************************************************
C     sorting by 'awadachi' method

      SUBROUTINE BUBBLE(NN,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN)
 
   30 CONTINUE
      INDEX=0
      DO 20 N=1,NN-1
      IF(E(N).GT.E(N+1)) THEN
      ED=E(N)
      E(N)=E(N+1)
      E(N+1)=ED
      INDEX=1      
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETUXY(J,JG,NEG,NWG,NRS,EEG,EWG,ERS, EPS,F, UX,UYP,UYN)

      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION EEG(NEG),EWG(NWG),ERS(NRS)
      DIMENSION UYP(0:JG,NEG),UYN(0:JG,NWG+NRS)

**      UX=F/EPS
      UX=1/SQRT(EPS)

      DO 20 K=1,NEG
        UYP(J,K)=EEG(K)
   20 CONTINUE
      DO 30 K=1,NWG
        UYN(J,K)=EWG(K)
   30 CONTINUE
      DO 40 K=1,NRS
        UYN(J,K+NWG)=ERS(K)
   40 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(JG,KMM,UX,UY)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z)
      DIMENSION UX(0:JG),UY(0:JG,KMM,0:1)
      CHARACTER CSGI*1,CS*5,CEPS*9

      CS='(*2o)'
      CS(2:2)=CSGI(5)
      CS(4:4)=CSGI(151)
      CEPS='#(gh)/2#a'
      CEPS(1:1)=CSGI(210)
      CEPS(8:8)=CSGI(151)
      UXMIN=10.**( IRLE( LOG10(UX(0)) ) )
      UXMAX=10.**( IRGE( LOG10(UX(JG)) ) )
      UYMIN=1D-3
      UYMAX=1D2
 
      CALL GRFRM
      CALL SGSVPT(0.18,0.63,0.15,0.9)
      CALL SGSWND(UXMIN,UXMAX,UYMIN,UYMAX)
      CALL SGSTRN(4)
      CALL SGSTRF
      CALL ULXLOG('T',1,9)
      CALL ULXLOG('B',1,9)
      CALL ULYLOG('L',1,9)
      CALL ULYLOG('R',1,9)
      CALL UYSTTL('L','Frequency '//CS,0.0)
**      CALL UXSTTL('B','Equivalent depth(km)',0.0)
        CALL UXSTTL('B',CEPS,0.0)
        DO 10 K=1,KMM
          CALL SGPLU(JG+1,UX,UY(0,K,0))
          CALL SGPLU(JG+1,UX,UY(0,K,1))
   10   CONTINUE
      RETURN

      ENTRY GYTTL(IY)
      IF(IY.EQ.1) THEN
        CALL UXMTTL('T','Eastward',0.0)
      ELSE
        CALL UXMTTL('T','Westward',0.0)
      ENDIF
      RETURN
      END
*********************************************************************
      SUBROUTINE SETPAR(MM,M,IS,KM,NS,ID)
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      IE=MOD(MM-M,2)
      KM=3*((MM-M)/2+1)-(1-IE)*(1+IS)
      ID=IS
      NS=M
 
      IF(M.EQ.0) THEN
      KM=KM-(2-IS)
      ID=1-IS
      NS=M+1
      END IF
 
      RETURN
      END
*******************************************************************
C < distinguish eastward gravity, westward gravity and Rossby waves >
 
      SUBROUTINE CHOSE
     I     (ND1,E1,EPS,M,UMISS,NEG,NWG,NRS,
     O      EEG,EWG,ERS)

**C     Ei(i=1,2):eigen values (-1/eps)
C     Ei(i=1,2):eigen values (sgm)
C     S2GRVi(i=1,2):(freq. of pure gravity wave)**2
     
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E1(ND1)
      DIMENSION EEG(NEG),EWG(NWG),ERS(NRS)
      LOGICAL   LFIRST
      DATA      LFIRST/.TRUE./
      SAVE EWG1SV,LFIRST

**      IF(ND2.GT.ND1) THEN
**        WRITE(6,*) 'ERROR IN SUBROUTINE CHOOSE : ND2 MUST BE .LE. ND1'
**        STOP
**      ENDIF

C     < count number of negative sgm >

      DO 11 N=1,ND1
        IF(E1(N).LT.0) NN1=N
   11 CONTINUE

C     < set positive sgm >

      DO 30 N=1,NEG
        IF(NN1+N.LE.ND1) THEN
          EEG(N)=E1(NN1+N)
        ELSE
          EEG(N)=UMISS
        ENDIF
   30 CONTINUE

C     < set (negative sgm) *(-1) >
C     count the numbers of Rosby and Gravity wave and set them:

      SGRV=M/SQRT(EPS)
      NGRV1=0

      IF(LFIRST) THEN
        LFIRST=.FALSE.
        DO 61 N=1,NN1
          IF(-E1(N).GE.SGRV) NGRV1=NGRV1+1
   61   CONTINUE
      ELSE
        SE=1E30  !" any sufficiently large number
        DO 70 N=1,ND1
          IF( ABS(E1(N)-EWG1SV) .LE. SE ) THEN
            NGRV1=N
            SE=ABS(E1(N)-EWG1SV)
          ENDIF
   70   CONTINUE
      ENDIF

      EWG1SV=E1(NGRV1)

      DO 80 N=1,NWG
        IF(NGRV1-N+1.GE.1) THEN
          EWG(N)=-E1(NGRV1-N+1)
        ELSE
          EWG(N)=UMISS
        ENDIF
   80 CONTINUE
      DO 100 N=1,NRS
        IF(NGRV1+N.LE.NN1) THEN
          ERS(N)=-E1(NGRV1+N)
        ELSE
          ERS(N)=UMISS
        ENDIF
  100 CONTINUE

      RETURN
      END
