C     < Table >
C     Vertical Group Velocity calculated from
C     EP flux of Hough Functions for Given Zonal Wave Number
C     (Solve Eigen Function Problem for Frequency.)
C     Use expantion by Legenre assotiated function.
C     Accuracy(OACC) is checked by solving two Matrices of different 
C     truncation MM/MM2.
C
C     Parameters :
C       MM:maximum of (total wave number)-(zonal wave number)
C    +     for calculation 
C       M :zonal wave number
C     Variables :
C       E:eigen values which correspond equivalent deapth.
C    +     associated function for geopotential field.
C       SGM:normalized angular frequency s.t. SGM=1 for semidiurnal mode
C       NS:symmetric(=0) or antisymmetric(=1)
C       NN:dimesion of the matrix of coeficint
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      REAL*8 OACC,RE,H,RR,G,DTDZ,RN2
      PARAMETER(MM=75,KMM=3*(MM/2+1),LM=KMM*(KMM+1)/2)
      PARAMETER(M=4,NN1=MM/2+2,JG=300,UMISS=0.,RMISS=0D0,OACC=1D-2)
      PARAMETER(LMAX=5)
      PARAMETER(RE=6.37D6,H=7000.D0,RR=287.1D0,G=9.7D0,DTDZ=0D0)
      DIMENSION D(LM),VW(2*KMM),E(KMM),E2(KMM),EV(KMM,KMM),EVD(KMM)
      DIMENSION ES(3*LMAX),EVS(KMM,3*LMAX),CGZ(3*LMAX)
      DIMENSION A(M:MM,3*LMAX),B(M:MM,3*LMAX),C(M:MM,3*LMAX)
      DIMENSION UX(3*LMAX,JG),UYP(3*LMAX,JG,0:1),UYN(3*LMAX,JG,0:1)

      CALL SGPWSN
      READ(5,*) IWS

      MM2=0.7*MM
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      F=4*RE*RE*OME*OME/G
      RN=2D0*PI/300
      RN2=RN*RN
      RMN1=0.001D0
      RMN2=60D0
      UXMAX=RMN2

      CALL GLPSET('LMISS',.TRUE.)
      CALL GLPSET('RMISS',UMISS)

      DO 10 J=1,JG
        FRAC=1D0*(JG-J)/(JG-1)
        RMN=RMN1+(RMN2-RMN1)*FRAC
        RM=RN/(2*OME*RE)*RMN
        HH=RN2/G/( RM**2+1D0/(4*H*H) )
        EPS=F/HH
        DO 20 IS=0,1
          CALL SETPAR(MM,M,IS,KM,MS,ID)
          CALL MATRIX(D,M,KM,MS,ID,EPS)
          CALL DSEIG1(D,KM,E,EV,KMM,KMD,VW,ICON)
          CALL BOBBLE(KM,KMM,E,EV,EVD)
          CALL SETPAR(MM2,M,IS,KM2,MS,ID)
          CALL MATRIX(D,M,KM2,MS,ID,EPS)
          CALL DTRID1(D,KM2,VW(1),VW(KMM+1),ICON)
          CALL DTRQL(VW(1),VW(KMM+1),KM2,E2,KMD2,ICON)
          CALL BUBBLE(KM2,E2)
          CALL CHOOSE(KM,E,KM2,E2,EV,KMM,ES,EVS,LMAX,OACC,RMISS,EPS,M)
          CALL SETABC(M,MM,3*LMAX,KM,KMM,MS,ID,ES,EVS,A,B,C,EPS,RMISS)
          CALL SETCGZ(M,MM,3*LMAX,ES,A,B, CGZ, RM,EPS,RN2,RMISS)
          CALL SETUXY(3*LMAX,ES,CGZ,RMN,UMISS, UX(1,J),
     +         UYP(1,J,IS),UYN(1,J,IS))
   20   CONTINUE
            write(6,*) 'finished ',j,'/',jg
   10 CONTINUE
      CALL GROPEN(IWS)
      CALL SLMGN(0.0,0.0,0.0,0.19)
      CALL SLDIV('Y',3,1)
      CALL TITLE(M)
      CALL SLRAT(0.65,1.0)
      CALL GRAPH(3*LMAX,JG,UX,UYP,UXMAX,0.2)
      CALL OTHAXS( RN/(2*OME*RE),UXMAX,RN2,G,H )
      CALL GYTTL(1)
      CALL GRAPH(3*LMAX,JG,UX,UYN,UXMAX,0.2)
      CALL OTHAXS( RN/(2*OME*RE),UXMAX,RN2,G,H )
      CALL GYTTL(-1)
      CALL GRAPH(3*LMAX,JG,UX,UYN,UXMAX,0.02)
      CALL OTHAXS( RN/(2*OME*RE),UXMAX,RN2,G,H )
      CALL GYTTL(-2)
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE GROPEN(IWS)
 
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZPSET('LOFFSET',.TRUE.)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UDPSET('LMSG',.FALSE.)
 
      RETURN
      END
************************************************************************
      SUBROUTINE MATRIX(A,M,KM,MS,ID,EPS)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(KM*(KM+1)/2)
 
      R(N)=-1D0*M/(N*(N+1))
      S(N)=-SQRT(N*(N+1)/EPS)
      T(N)=SQRT(1D0*(N*N-1)*(N*N-M*M)/(4*N*N-1))/N
 
      DO 10 I=1,KM*(KM+1)/2
      A(I)=0
   10 CONTINUE
 
      N=MS
      L=1
      DO 20 K=1,KM
      JR=MOD(K-1-ID,3)
      IF(JR.NE.0) THEN
      A(L)=R(N)
      N=N+1
      END IF
      L=L+K+1
   20 CONTINUE
 
      N=MS+ID
      L=2
      DO 30 K=2,KM
      JS=MOD(K-2-ID,3)
      JT=MOD(K-ID,3)
      IF(JS.EQ.0) THEN
      A(L)=S(N)
      N=N+1
      ELSE IF(JT.EQ.0) THEN
      A(L)=T(N)
      N=N+1
      END IF
      L=L+K+1
   30 CONTINUE
 
      N=MS+2-ID
      L=4
      DO 40 K=3,KM
      JS=MOD(K-2-ID,3)
      IF(JS.EQ.0) THEN
      A(L)=T(N)
      N=N+2
      END IF
      L=L+K+1
   40 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE TITLE(M)
 
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER CS*3
 
      CS='s=#'
      WRITE(CS(3:3),'(I1)') M
 
      CALL SLSTTL('Hough Modes '//CS,'T',0.0,0.3,0.03,1)
      CALL SLSTTL('Vertical Group Velocity Derived from'
     +     ,'T',0.0,-0.5,0.027,2)
      CALL SLSTTL('Meridiornary Integrated EP Flux'
     +     ,'T',0.0,-1.0,0.027,3)
 
      RETURN
      END
************************************************************************
C     sorting by 'awadachi' method

      SUBROUTINE BUBBLE(NN,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN)
 
   30 CONTINUE
      INDEX=0
      DO 20 N=1,NN-1
      IF(E(N).GT.E(N+1)) THEN
      ED=E(N)
      E(N)=E(N+1)
      E(N+1)=ED
      INDEX=1      
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30
 
      RETURN
      END
********************************************************************** *
      SUBROUTINE BOBBLE(NN,KK,E,EV,EVD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN),EV(KK,NN),EVD(NN)
 
   30 CONTINUE
      INDEX=0
      DO 20 N=1,NN-1
      IF(E(N).GT.E(N+1)) THEN
      ED=E(N)
      E(N)=E(N+1)
      E(N+1)=ED
      DO 10 L=1,NN
      EVD(L)=EV(L,N)
      EV(L,N)=EV(L,N+1)
      EV(L,N+1)=EVD(L)
   10 CONTINUE
      INDEX=1      
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETABC(M,MM,LMX,NN,KK,MS,ID,E,EV,A,B,C,RMISS)

      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(LMX),EV(KK,LMX),A(M:MM,LMX),B(M:MM,LMX),C(M:MM,LMX)
 
      F(N)=1/SQRT(1D0*N*(N+1))
 
      DO 40 L=1,LMX
      
      DO 10 N=M,MM
      A(N,L)=0
      B(N,L)=0
      C(N,L)=0
   10 CONTINUE
 
      IF(ID.EQ.0 .AND. E(L).NE.RMISS) THEN
 
      N=MS
      DO 20 ND=1,NN,3
      C(N,L)=EV(ND,L)
      N=N+2
   20 CONTINUE
      N=MS
      DO 21 ND=2,NN,3
      A(N,L)=EV(ND,L)*F(N)
      N=N+2
   21 CONTINUE
      N=MS+1
      DO 22 ND=3,NN,3
      B(N,L)=EV(ND,L)*F(N)
      N=N+2
   22 CONTINUE
 
      ELSE IF(ID.EQ.1 .AND. E(L).NE.RMISS) THEN
 
      N=MS
      DO 30 ND=1,NN,3
      B(N,L)=EV(ND,L)*F(N)
      N=N+2
   30 CONTINUE
      N=MS+1
      DO 31 ND=2,NN,3
      C(N,L)=EV(ND,L)
      N=N+2
   31 CONTINUE
      N=MS+1
      DO 32 ND=3,NN,3
      A(N,L)=EV(ND,L)*F(N)
      N=N+2
   32 CONTINUE
 
      END IF
 
   40 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETUXY(KM,ES,E,X,UMISS, UX,UYP,UYN)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION ES(KM),E(KM),UX(KM),UYP(KM),UYN(KM)

      DO 10 K=1,KM
        UX(K)=X
        UYP(K)=UMISS
        UYN(K)=UMISS
   10 CONTINUE
      DO 20 K=1,KM
      ED=E(K)
      IF(ES(K).LT.0D0) THEN
        UYN(K)=ED
      ELSE IF(ES(K).GT.0D0) THEN
        UYP(K)=ED
      END IF
   20 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(KMM,JG,UX,UY,UXMAX,UYMAX)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION UX(KMM,JG),UY(KMM,JG,0:1),ULB(4),USLB(25)
      CHARACTER CSGI*1,CS*20,CEPS*20,CLB(4)*5,CLBD*5

      CS='(2oa/N)k_z"'
      CS(3:3)=CSGI(151)
      CEPS='C_gz"(ms|-1")'
      UXMIN=0.
      UYMIN=0.

      CALL GRFRM
      CALL SGLSET('LCLIP',.TRUE.)
      CALL GRSVPT(0.19,0.54,0.43,0.89)
      CALL GRSWND(UXMIN,UXMAX,UYMIN,UYMAX)
      CALL GRSTRN (1)
      CALL GRSTRF
      CALL USDAXS
      CALL UXMTTL('B',CS,0.0)
      CALL UYMTTL('L',CEPS,0.0)
      CALL SGSPMS(0.01)
      CALL SGSPMT(1)
      CALL SGPMU(KMM*JG,UX,UY(1,1,0))
      CALL SGSPMS(0.0025)
      CALL SGSPMT(7)
      CALL SGPMU(KMM*JG,UX,UY(1,1,1))
      RETURN

c     < other axies scale DM(x_axis) DMY(y-axis) >

      ENTRY OTHAXS(DM,UXMAX,RN2,G,H)
      UDM=DM
      UDV=UXMAX*UDM/6
      CALL UZPGET('XFACT',UF)
      CALL UZPSET('XFACT',UDM)
**      CALL UXSAXS('B')
**      CALL UZFACT(0.7)
**      CALL UXAXDV('B',UDV,2*UDV)
**      CALL UZFACT(1/0.7)
**      CALL UXSTTL('B','k_z" (m|-1")',0.0)
      DO 10 K=1,4
        ULB(K)=2*UDV*(K-1)
        IF(K.NE.1) THEN
        WRITE(CLB(K)(1:5),'(F5.1)') 2*3.141593/ULB(K)/1000.
        ENDIF
   10 CONTINUE
      DO 20 K=1,7
        USLB(K)=UDV*(K-1)
   20 CONTINUE
      CLB(1)=CSGI(189)
      CALL UXSAXS('B')
      CALL UXAXLB('B',USLB,7,ULB,CLB,5,4)
      CALL UXSTTL('B',CSGI(162)//'_z" (km)',0.0)
      DO 30 K=1,4
        ULB(K)=2*UDV*(K-1)
        HH = RN2/G/( (ULB(K))**2+1D0/(4*H*H) ) / 1000D0
        IF(HH.GE.1) THEN
          WRITE(CLB(K)(1:4),'(F4.2)') HH
        ELSE
          WRITE(CLBD(1:5),'(F5.3)') HH
          CLB(K)=CLBD(2:5)
        ENDIF
***        CALL CHVAL('C',REAL(HH),CLB(K))
   30 CONTINUE
      CALL UXSAXS('B')
      CALL UXAXLB('B',USLB,7,ULB,CLB,5,4)
      CALL UXSTTL('B','h (km)',0.0)
      CALL UZPSET('XFACT',UF)
      RETURN
      
      ENTRY GYTTL(IY)
      IF(IY.EQ.1) THEN
        CALL UXMTTL('T','Eastward',0.0)
      ELSE IF(IY.EQ.-1) THEN
        CALL UXMTTL('T','Westward',0.0)
      ELSE
        CALL UXSTTL('T',' ',0.0)
**        CALL UXSTTL('T','Westward(slow)',0.0)
        CALL UXSTTL('T','Westward(magnified)',0.3)
      ENDIF
      RETURN
      END
*********************************************************************
      SUBROUTINE SETPAR(MM,M,IS,KM,MS,ID)
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      IE=MOD(MM-M,2)
      KM=3*((MM-M)/2+1)-(1-IE)*(1+IS)
      ID=IS
      MS=M
 
      IF(M.EQ.0) THEN
      KM=KM-(2-IS)
      ID=1-IS
      MS=M+1
      END IF
 
      RETURN
      END
*******************************************************************
C     < choose reliable eigen values order OACC>
 
      SUBROUTINE CHOOSE(ND1,E1,ND2,E2,EV,KMM,ES,EVS,LMAX,OACC,
     +     RMISS,EPS,M)

C     Ei(i=1,2):eigen values (sgm)
C     SGRV:freq. of pure gravity wave
     
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E1(ND1),E2(ND2),EV(KMM,ND1),ES(3*LMAX),EVS(KMM,3*LMAX)

      IF(ND2.GT.ND1) THEN
        WRITE(6,*) 'ERROR IN SUBROUTINE CHOOSE : ND2 MUST BE .LE. ND1'
        STOP
      ENDIF
      DO 5 N=1,3*LMAX
        ES(N)=RMISS
    5 CONTINUE

C     < count the number of negative sgm (Eastward Gravity wave) >

      DO 11 N=1,ND1
        IF(E1(N).LT.0) NN1=N
   11 CONTINUE
      DO 12 N=1,ND2
        IF(E2(N).LT.0) NN2=N
   12 CONTINUE

C     < count the numbers of Westward Gravity and Rosby wave >

      SGRV=M/SQRT(EPS)
      NGRV1=0
      DO 61 N=1,NN1
        IF(-E1(N).GE.SGRV) NGRV1=NGRV1+1
   61 CONTINUE
      NGRV2=0
      DO 62 N=1,NN2
        IF(-E2(N).GE.SGRV) NGRV2=NGRV2+1
   62 CONTINUE
      NROS1=NN1-NGRV1
      NROS2=NN2-NGRV2

C     < set positive accurate sgm (Eastward Gravity wave) >

      NAP=0
      DO 20 N=1,MIN(ND1-NN1,ND2-NN2)
        A1=ABS(E1(NN1+N))
        A2=ABS(E2(NN2+N))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAP=NAP+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAP=NAP+1
        ELSE
          NAP=NAP+1
        ENDIF
   20 CONTINUE
      DO 30 N=1,MIN(NAP,LMAX)
        ES(N)=E1(NN1+N)
        DO 32 K=1,ND1
        EVS(K,N)=EV(K,NN1+N)
   32   CONTINUE
   30 CONTINUE

C     < set negative accurate eps (Westward Gravity wave) >

      NAG=0
      DO 70 N=1,MIN(NGRV1,NGRV2)
        A1=ABS(E1(NGRV1-N+1))
        A2=ABS(E2(NGRV2-N+1))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAG=NAG+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAG=NAG+1
        ELSE
          NAG=NAG+1
        ENDIF
   70 CONTINUE
      DO 80 N=1,MIN(NAG,LMAX)
        ES(LMAX+N)=E1(NGRV1-N+1)
        DO 82 K=1,ND1
        EVS(K,LMAX+N)=EV(K,NGRV1-N+1)
   82   CONTINUE
   80 CONTINUE

C     < set negative accurate eps (Westward Rosby wave) >

      NAR=0
      DO 90 N=1,MIN(NROS1,NROS2)
        A1=ABS(E1(NGRV1+N))
        A2=ABS(E2(NGRV2+N))
        ERR=ABS(A1-A2)
        IF(A1.NE.0) THEN
          IF(ERR/A1.LT.OACC) NAR=NAR+1
        ELSE IF(A2.NE.0) THEN
          IF(ERR/A2.LT.OACC) NAR=NAR+1
        ELSE
          NAR=NAR+1
        ENDIF
   90 CONTINUE
      DO 100 N=1,MIN(NAR,LMAX)
        ES(2*LMAX+N)=E1(NGRV1+N)
        DO 102 K=1,ND1
       EVS(K,2*LMAX+N)=EV(K,NGRV1+N)
  102   CONTINUE
  100 CONTINUE

      RETURN
      END
******************************************************************
C     < calculate meridiornary integrated EP flux >

      SUBROUTINE SETCGZ(M,MM,LMX,E,A,B, CGZ, RM,EPS,RN2,RMISS)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(LMX),CGZ(LMX),A(M:MM,LMX),B(M:MM,LMX)

      PI=4*ATAN(1D0)
      RE=6.37D6
      OME=2*PI/86400

      DO 10 L=1,LMX
        SGM=E(L)
        IF(SGM.NE.RMISS) THEN
        CALL ENERG(M,MM,A(M,L),B(M,L),ENG, EPS,SGM)
        TIMES=-(2*OME)**2/RN2*RE*ABS(RM/SGM)/EPS/ENG
        CALL INTEP(M,MM,A(M,L),B(M,L), SEP, TIMES,SGM)
        CALL CGZZ(SEP,CGZ(L), OME,SGM,RE,M)
        CALL GLPGET('RMISS',UMISS)
        IF (CGZ(L).LT.0D0) THEN
         WRITE(6,*) '(SETCGZ) Negative group velocity was removed where'
         WRITE(6,*) '    EPFLUX,,SGM,CGZ;',SEP,SGM,CGZ(L)
         CGZ(L)=UMISS  
        ENDIF
        ENDIF
   10 CONTINUE

      RETURN
      END
******************************************************************
C     < Meridiornal integration of EP flux >

      SUBROUTINE INTEP(M,MM,A,B, SEP, TIMES,SGM)

      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(M:MM),B(M:MM)
      R(N)=SQRT(1D0*(N*N-M*M)/(4*N*N-1))
      E(N)=R(N-1)*R(N)
      F(N)=M*SGM-(N+1)*(R(N))**2+N*(R(N+1))**2
      
      SEP=0
      DO 10 N=M+2,MM-2
      SEP=SEP+N*(N+1)*( (N-2)*E(N)*A(N-2)*A(N)+F(N)*A(N)*A(N)
     +    -(N+3)*E(N+2)*A(N)*A(N+2)-(M+SGM*(N-1))*R(N)*B(N-1)*A(N)
     +    -(M-SGM*(N+2))*R(N+1)*B(N+1)*A(N) )
   10 CONTINUE
      IF(MM-M.LT.3) THEN
        WRITE(6,*) 'ERROR(INTEP) MM-M.LT.3'
        STOP
      ENDIF
      N=M
      SEP=SEP+N*(N+1)*( F(N)*A(N)*A(N)
     +    -(N+3)*E(N+2)*A(N)*A(N+2)-(M-SGM*(N+2))*R(N+1)*B(N+1)*A(N) )
      N=M+1
      SEP=SEP+N*(N+1)*( F(N)*A(N)*A(N)
     +    -(N+3)*E(N+2)*A(N)*A(N+2)-(M+SGM*(N-1))*R(N)*B(N-1)*A(N)
     +    -(M-SGM*(N+2))*R(N+1)*B(N+1)*A(N) )
      N=MM-1
      SEP=SEP+N*(N+1)*( (N-2)*E(N)*A(N-2)*A(N)+F(N)*A(N)*A(N)
     +    -(M+SGM*(N-1))*R(N)*B(N-1)*A(N)
     +    -(M-SGM*(N+2))*R(N+1)*B(N+1)*A(N) )
      N=MM
      SEP=SEP+N*(N+1)*( (N-2)*E(N)*A(N-2)*A(N)+F(N)*A(N)*A(N)
     +    -(M+SGM*(N-1))*R(N)*B(N-1)*A(N) )

      SEP=TIMES*SEP
      RETURN
      END
*******************************************************************
c     < set wave energy >

      SUBROUTINE ENERG(M,MM,A,B,ENG, EPS,SGM)

      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(M:MM),B(M:MM)
 
      ENGP=0
      ENGK=0
      DO 60 N=M,MM
      ENGP=ENGP+(N*(N+1)*A(N)/SGM)**2/ABS(EPS)
      ENGK=ENGK+N*(N+1)*(A(N)*A(N)+B(N)*B(N))
   60 CONTINUE
      ENG=1D0/2*(ENGP+ENGK)
      ENGS=SQRT(ENG)
      
      RETURN
      END
*********************************************************************
C     < calculate vertical group velocity from m.integrated Ep flux >

      SUBROUTINE CGZZ(SEP,CGZ, OME,SGM,RE,M)

C     SEP:Meridiornary integrated Ep flux
C     CGZ:vertical group velocity
C     CX :zonal phase velocity

      IMPLICIT REAL*8(A-H,O-Z)
      CX=2*OME*SGM*RE/M
      CGZ=-CX*SEP
      RETURN
      END
