***********************************************************************
*     Hough Functions of Migrating Tide of Zonal Wave Number 'M'. 
*     (Eigen function problem for equivalent depth)
*     Plot geopotential field (=Hough Function) as function of latitude.
*       original   Ishioka,K.
*       last revised   Mar 1993   Horinouchi,T. 
*       (addapted to dcl-5, Apr 1997, Horiouchi)
***********************************************************************
c     parameters
c       MM: maximum total wave number for calculation 
c       JH: meridional resolution (grid number for hemisphere) for plot
c       M : zonal wave number ( .ge.1 )
c       NS: symmetric(=0) or antisymmetric(=1)
c     variables
c       D,SD: matrix components
c       E : eigen values ( = -g*h/(2*ome*Re)**2 )
c       EV: eigen vectors
*======================================================================
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      PARAMETER(JH=90,MM=100,M=1,NS=0,NE=(MM+NS-M)/2,NN=NE-NS+1)
      DIMENSION D(NN),SD(NN),E(NN),EV(NN,NN)
      DIMENSION X(0:JH,2),PD(0:JH,2),P(0:JH,2,NN)
      DIMENSION UX(-JH:JH),UY(-JH:JH)
 
      CALL MATRIX(M,NS,NN,D,SD)
      CALL DTEIG1(D,SD,NN,E,EV,NN,MD,ICON)
      CALL CHANGE(NN,E)
      CALL LEGEND(M,NS,NN,JH,X,PD,P)
      CALL SGPWSN
      READ(5,*) IWS
      CALL GROPEN(-IWS)
      CALL SLMGN(0.0,0.0,0.0,0.05)
      CALL TITLE(M,NS)
      CALL SLDIV('T',2,4)
      CALL SLRAT(1.0,0.7)
* --- plot positive equivalent depth
      DO 10 L=NN,NN-3,-1
      CALL GRAPH(NS,NN,JH,L,P,EV,UX,UY)
      CALL INFORM(NN,L,E)
   10 CONTINUE
* --- plot negative equivalent depth
      DO 11 L=1,4
      CALL GRAPH(NS,NN,JH,L,P,EV,UX,UY)
      CALL INFORM(NN,L,E)
   11 CONTINUE
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE GROPEN(IWS)
 
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UDPSET('LMSG',.FALSE.)
 
      RETURN
      END
************************************************************************
      SUBROUTINE CHANGE(NN,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN)
 
      A=6.37D6
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      G=9.8D0
      F=4*A*A*OME*OME/G*1D-3
 
      DO 20 ND=1,NN
      E(ND)=-F*E(ND)
   20 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE MATRIX(M,NS,NN,D,SD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION D(NN),SD(NN)
 
      R(N)=SQRT(1D0*(N*N-M*M)/(4*N*N-1))
      S(N)=(N+2)*(N-1)
      T1(N)=R(N)*R(N)*(N-1)*(N-1)/(S(N-1)*N*N)
      T2(N)=S(N)*M*M/(4D0*N*N*(N+1)*(N+1))
      T3(N)=R(N+1)*R(N+1)*(N+2)*(N+2)/(S(N+1)*(N+1)*(N+1))
 
      DO 10 ND=2,NN
      N=M+NS+ND*2-2
      D(ND)=T1(N)-T2(N)+T3(N)
      N=N-1
      if(n.ne.1) SD(ND)=R(N)*R(N+1)/S(N)
   10 CONTINUE
 
      IF(NS.EQ.0) THEN
        N=M
        D(1)=-T2(N)+T3(N)
      ELSE
        IF(M.EQ.1) THEN
          D(1)=0
          SD(2)=0
        ELSE
          N=M+1
          D(1)=T1(N)-T2(N)+T3(N)
        END IF
      END IF
 
      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(NS,NN,JH,L,P,EV,UX,UY)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION EV(NN,NN)
      DIMENSION P(0:JH,2,NN)
      DIMENSION UX(-JH:JH),UY(-JH:JH)
 
      DO 10 J=-JH,JH
      UX(J)=90.0*J/JH
      UY(J)=0
   10 CONTINUE
 
      DO 20 ND=1,NN
      DO 20 J=0,JH
      UY(J)=UY(J)+EV(ND,L)*P(J,1,ND)
   20 CONTINUE
 
      IF(NS.EQ.0) THEN
      DO 30 J=0,JH
      UY(-J)=UY(J)
   30 CONTINUE
      ELSE
      DO 40 J=0,JH
      UY(-J)=-UY(J)
   40 CONTINUE
      END IF
 
      UYMAX=1E-10
      DO 50 J=0,JH
      UYA=ABS(UY(J))
      IF(UYA.GT.UYMAX) THEN
      UYMAX=UYA
      F=UY(J)/UYA
      END IF
   50 CONTINUE
 
      IF(F.LT.0) THEN
      DO 60 J=-JH,JH
      UY(J)=-UY(J)
   60 CONTINUE
      END IF
 
      CALL GRFRM
      CALL GRSVPT(0.1,0.9,0.1,0.6)
      UX1=-90
      UX2=90
      UY1=-UYMAX*1.1
      UY2=UYMAX*1.1
      CALL GRSWND(UX1,UX2,UY1,UY2)
      CALL GRSTRN (1)
      CALL GRSTRF
      CALL UXAXDV('B',10.0,30.0)
      CALL UXAXDV('T',10.0,30.0)
      CALL UYPAXS('L',1)
      CALL UYPAXS('R',1)
      CALL UXPAXS('U',1)
      CALL UYPAXS('U',1)
      CALL SGPLU(JH*2+1,UX,UY)
 
      RETURN
      END
************************************************************************
      SUBROUTINE INFORM(NN,L,E)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(NN)
      CHARACTER CH*15
 
      CH(1:2)='h='
      CH(12:15)='(km)'
      WRITE(CH(3:11),'(F9.5)') E(L)
 
      CALL UXMTTL('T',CH,0.0)
 
      RETURN
      END
************************************************************************
      SUBROUTINE TITLE(M,NS)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      CHARACTER CS*3,CT*24
 
      CS(1:2)='s='
      WRITE(CS(3:3),'(I1)') M
 
      CT=' (symmetric)'
      IF(NS.EQ.1) CT=' (antisymmetric)'
 
      CALL SLSTTL('Migrating Tides '//CS//CT,'T',0.0,0.0,0.02,1)
 
      RETURN
      END
************************************************************************
      SUBROUTINE LEGEND(M,NS,NN,JH,X,PD,P)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(0:JH,2),PD(0:JH,2),P(0:JH,2,NN)
 
      PAI=4*ATAN(1D0)
 
      DO 10 J=0,JH
      X(J,1)=SIN(PAI/2*J/JH)
      X(J,2)=COS(PAI/2*J/JH)
      PD(J,1)=1
   10 CONTINUE
 
      DO 20 MD=1,M
      DO 20 J=0,JH
      PD(J,1)=SQRT(1D0*(2*MD+1)/(2*MD))*X(J,2)*PD(J,1)
   20 CONTINUE
      DO 21 J=0,JH
      PD(J,2)=-M*X(J,1)*PD(J,1)
   21 CONTINUE
 
      IF(NS.EQ.0) THEN
      DO 30 J=0,JH
      P(J,1,1)=PD(J,1)
      P(J,2,1)=PD(J,2)
   30 CONTINUE
      ELSE
      N=M+1
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 31 J=0,JH
      P(J,1,1)=E1*(N*X(J,1)*PD(J,1)-PD(J,2))
      P(J,2,1)=E2*PD(J,1)-N*X(J,1)*P(J,1,1)
   31 CONTINUE
      END IF
 
      DO 41 ND=2,NN
      N=M+NS+ND*2-3
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 40 J=0,JH
      PD(J,1)=E1*(N*X(J,1)*P(J,1,ND-1)-P(J,2,ND-1))
      PD(J,2)=E2*P(J,1,ND-1)-N*X(J,1)*PD(J,1)
   40 CONTINUE
      N=N+1
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 41 J=0,JH
      P(J,1,ND)=E1*(N*X(J,1)*PD(J,1)-PD(J,2))
      P(J,2,ND)=E2*PD(J,1)-N*X(J,1)*P(J,1,ND)
   41 CONTINUE
 
      RETURN
      END





