***********************************************************************
*     Hough Functions for Free Oscillation in Isothermal Atmosphere 
*     of Zonal Wave Number 'M'.
*     (Eigen function problem for frequency)
*     Plot geopotential field (=Hough Function) as function of latitude.
*       original   Ishioka,K.
*       last revised   Mar 1993   Horinouchi,T. 
*       (addapted to dcl-5, Apr 1997, Horiouchi)
***********************************************************************
c     parameters
c       MM: maximum total wave number for calculation 
c       JH: meridional resolution(grid number for hemisphere) to plot
c    +      geopotential field (should be .le.MM)
c       M : zonal wave number
c       NS: symmetric(=0) or antisymmetric(=1) 
c       H : scale height
c       HH: equivalent depth (=1.4D0*H)
c     variables
c       E: eigen values which correspond nondimensional frequency 
c       EV: eigen vectors 
c       A,B,C: coefficients for velocity potential, stream function,
c    +     geopotential.
c       P(*,1,N): Legendre associated func. zonal w.n. M total wn N
c       P(*,2,N):(1-x**2)(d/dx)P(*,1,N)
*====================================================================== 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      PARAMETER(JH=90,MM=60,NND=3*(MM/2+1))
      DIMENSION D(NND*(NND+1)/2),E(NND),EV(NND,NND),VW(2*NND)
      DIMENSION A(0:MM,NND),B(0:MM,NND),C(0:MM,NND)
      DIMENSION X(0:JH,2),P(0:JH,2,0:MM)
      DIMENSION UX(-JH:JH),UY(-JH:JH)
 
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      H=8D3
      HH=1.4D0*H
         write(6,*) 'Equivalent depth [m] ?'
         read(5,*) hh
      R=6.37D6
      G=9.8D0
      EPS=4*R*R*OME*OME/(G*HH)
      WRITE(6,*) 'Zonal Wave Number ?'
      READ(5,*) M
      WRITE(6,*) 'Symmetric(=0) or Antisymmetric(=1) ?'
      READ(5,*) NS
      MS=MM-M-(MM-M)/2*2
      NN=3*((MM-M)/2+1)-(1-MS)*(NS+1)
      CALL SGPWSN
      READ(5,*) IWS
      IWS=-IWS

      CALL MATRIX(M,NS,NN,EPS,D)
      CALL DSEIG1(D,NN,E,EV,NND,MD,VW,ICON)
      CALL BUBBLE(NN,NND,E,EV,VW)
      CALL SETABC(M,MM,NN,NS,EV,NND,A,B,C)
      CALL LEGEND(M,MM,JH,X,P)
      CALL CHANGE(NN,E)
 
      CALL GROPEN(IWS)
      CALL SLMGN(0.0,0.0,0.0,0.05)
      CALL TITLE(M,NS)
      CALL SLDIV('T',2,4)
      CALL SLRAT(1.0,0.7)

**      DO 15 L=1,NN
**      CALL GRAPH(M,MM,JH,NN,L,P,C,UX,UY)
**      CALL INFORM(NN,L,E)
**   15 CONTINUE
* --- plot Rossby waves
      INDEX=1
      L=1
   10 CONTINUE
      ED=E(L)
      IF(ED.GT.-1) THEN
        IF(ED.LT.-1D-5) THEN
          CALL GRAPH(M,MM,JH,NN,L,P,C,UX,UY)
          CALL INFORM(NN,L,E)
           write(6,*) c(2,l),c(4,l),c(6,l)
        ENDIF
        INDEX=INDEX+1
      END IF
      L=L+1
      IF(INDEX.LE.4) GOTO 10
* --- plot westward gravity waves
      DO 20 I=1,2
        LD=L-4-I
        CALL GRAPH(M,MM,JH,NN,LD,P,C,UX,UY)
        CALL INFORM(NN,LD,E)
         write(6,*) c(2,ld),c(4,ld),c(6,ld)
   20 CONTINUE
      INDEX=1
* --- plot eastward gravity waves
   30 CONTINUE
      ED=E(L)
      IF(ED.GT.1D-5) THEN
        CALL GRAPH(M,MM,JH,NN,L,P,C,UX,UY)
        CALL INFORM(NN,L,E)
        INDEX=INDEX+1
      END IF
      L=L+1
      IF(INDEX.LE.2) GOTO 30 

      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE CHANGE(NN,E)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(NN)
 
      DO 10 ND=1,NN
      E(ND)=2*E(ND)
   10 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETABC(M,MM,NN,NS,EV,NND,A,B,C)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION EV(NND,NN),A(0:MM,NN),B(0:MM,NN),C(0:MM,NN)
 
      F(N)=1/SQRT(1D0*N*(N+1))
 
      DO 40 L=1,NN
 
      DO 10 N=0,MM
      A(N,L)=0
      B(N,L)=0
      C(N,L)=0
   10 CONTINUE
 
      IF(NS.EQ.0) THEN
 
      N=M
      DO 20 ND=1,NN,3
      C(N,L)=EV(ND,L)
      N=N+2
   20 CONTINUE
      N=M
      DO 21 ND=2,NN,3
      A(N,L)=EV(ND,L)*F(N)
      N=N+2
   21 CONTINUE
      N=M+1
      DO 22 ND=3,NN,3
      B(N,L)=EV(ND,L)*F(N)
      N=N+2
   22 CONTINUE
 
      ELSE
 
      N=M
      DO 30 ND=1,NN,3
      B(N,L)=EV(ND,L)*F(N)
      N=N+2
   30 CONTINUE
      N=M+1
      DO 31 ND=2,NN,3
      C(N,L)=EV(ND,L)
      N=N+2
   31 CONTINUE
      N=M+1
      DO 32 ND=3,NN,3
      A(N,L)=EV(ND,L)*F(N)
      N=N+2
   32 CONTINUE
 
      END IF
 
   40 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE GROPEN(IWS)
 
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UDPSET('LMSG',.FALSE.)
 
      RETURN
      END
********************************************************************** *
c     < sorting >

      SUBROUTINE BUBBLE(NN,KK,E,EV,EVD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN),EV(KK,NN),EVD(KK)
 
   30 CONTINUE
      INDEX=0
      DO 20 N=1,NN-1
      IF(E(N).GT.E(N+1)) THEN
      ED=E(N)
      E(N)=E(N+1)
      E(N+1)=ED
      DO 10 L=1,KK
      EVD(L)=EV(L,N)
      EV(L,N)=EV(L,N+1)
      EV(L,N+1)=EVD(L)
   10 CONTINUE
      INDEX=1      
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30
 
      RETURN
      END
************************************************************************
      SUBROUTINE MATRIX(M,NS,NN,EPS,D)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION D(NN*(NN+1)/2)
 
      R(N)=-1D0*M/(N*(N+1))
      S(N)=-SQRT(N*(N+1)/EPS)
      T(N)=SQRT(1D0*(N*N-1)*(N*N-M*M)/(4*N*N-1))/N
 
      KM=NN*(NN+1)/2
 
      DO 10 K=1,KM
      D(K)=0
   10 CONTINUE
 
      IF(NS.EQ.0) THEN
 
      MD=M
      ND=1
      DO 20 N=1,NN
        NOD=MOD(N,3)
        IF(NOD.NE.1) THEN
          IF(MD.NE.0) THEN
            D(ND)=R(MD)
          ELSE
            D(ND)=0
          ENDIF
          MD=MD+1
        END IF
        ND=ND+N+1
   20 CONTINUE
      MD=M
      ND=2
      DO 21 N=2,NN
        NOD=MOD(N,3)
        IF(NOD.EQ.2) THEN
          D(ND)=S(MD)
          MD=MD+1
        ELSE IF(NOD.EQ.0) THEN
          D(ND)=T(MD)
          MD=MD+1
        ENDIF
        ND=ND+N+1
   21 CONTINUE
      MD=M+2
      ND=4
      DO 22 N=3,NN
        NOD=MOD(N,3)
        IF(NOD.EQ.2) THEN
          D(ND)=T(MD)
          MD=MD+2
        ENDIF
        ND=ND+N+1
   22 CONTINUE
      
      ELSE
        
      MD=M
      ND=1
      DO 30 N=1,NN
        NOD=MOD(N,3)
        IF(NOD.NE.2) THEN
          IF(MD.NE.0) THEN
            D(ND)=R(MD)
          ELSE
            D(ND)=0
          ENDIF
          MD=MD+1
        END IF
        ND=ND+N+1
   30 CONTINUE
      MD=M+1
      ND=2
      DO 31 N=2,NN
        NOD=MOD(N,3)
        IF(NOD.EQ.0) THEN
          D(ND)=S(MD)
          MD=MD+1
        ELSE IF(NOD.EQ.1) THEN
          D(ND)=T(MD)
          MD=MD+1
        END IF
        ND=ND+N+1
   31 CONTINUE
      MD=M+1
      ND=4
      DO 32 N=3,NN
        NOD=MOD(N,3)
        IF(NOD.EQ.0) THEN
          D(ND)=T(MD)
        ELSE
          D(ND)=0
          MD=MD+2
        END IF
        ND=ND+N+1
   32 CONTINUE
      
      END IF
 
      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(M,MM,JH,NN,L,P,C,UX,UY)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION C(0:MM,NN)
      DIMENSION P(0:JH,2,0:MM)
      DIMENSION UX(-JH:JH),UY(-JH:JH)
 
      DO 10 J=-JH,JH
      UX(J)=90.0*J/JH
      UY(J)=0
   10 CONTINUE
 
      DO 20 N=M,MM,2
      DO 20 J=0,JH
      UY(J)=UY(J)+C(N,L)*P(J,1,N)
   20 CONTINUE
      DO 30 J=1,JH
      UY(-J)=UY(J)
   30 CONTINUE
 
      DO 40 N=M+1,MM,2
      DO 40 J=1,JH
      UY(J)=UY(J)+C(N,L)*P(J,1,N)
      UY(-J)=UY(-J)-C(N,L)*P(J,1,N)
   40 CONTINUE
 
      UYMAX=1E-10
      DO 50 J=0,JH
      UYA=ABS(UY(J))
      IF(UYA.GT.UYMAX) THEN
      UYMAX=UYA
      F=UY(J)/UYA
      END IF
   50 CONTINUE
 
      IF(F.LT.0) THEN
      DO 60 J=-JH,JH
      UY(J)=-UY(J)
   60 CONTINUE
      END IF
 
      CALL GRFRM
      CALL GRSVPT(0.1,0.9,0.1,0.6)
      UX1=-90
      UX2=90
      UY1=-UYMAX*1.1
      UY2=UYMAX*1.1
      CALL GRSWND(UX1,UX2,UY1,UY2)
      CALL GRSTRN (1)
      CALL GRSTRF
      CALL UXAXDV('B',10.0,30.0)
      CALL UXAXDV('T',10.0,30.0)
      CALL UYPAXS('L',1)
      CALL UYPAXS('R',1)
      CALL UXPAXS('U',1)
      CALL UYPAXS('U',1)
      CALL SGPLU(JH*2+1,UX,UY)
 
      RETURN
      END
************************************************************************
      SUBROUTINE INFORM(NN,L,E)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(NN)
      CHARACTER CS*15,CSGI*1
 
      CS(1:2)=CSGI(169)//'='
      CS(12:15)='('//CSGI(5)//CSGI(151)//')'
      WRITE(CS(3:11),'(F9.5)') E(L)
 
      CALL UXMTTL('T',CS,0.0)
 
      RETURN
      END
************************************************************************
      SUBROUTINE TITLE(M,NS)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      CHARACTER CS*3,CT*24
 
      CS(1:2)='s='
      WRITE(CS(3:3),'(I1)') M
 
      CT='  (symmetric)'
      IF(NS.EQ.1) CT=' (antisymmetric)'
 
      CALL SLSTTL('Free Oscillations '//CS//CT,'T',0.0,0.0,0.02,1)
 
      RETURN
      END
************************************************************************
      SUBROUTINE LEGEND(M,MM,JH,X,P)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(0:JH,2),P(0:JH,2,0:MM)
 
      PAI=4*ATAN(1D0)
 
      DO 10 J=0,JH
      X(J,1)=SIN(PAI/2*J/JH)
      X(J,2)=COS(PAI/2*J/JH)
      P(J,1,M)=1
   10 CONTINUE
 
      DO 20 MD=1,M
      DO 20 J=0,JH
      P(J,1,M)=SQRT(1D0*(2*MD+1)/(2*MD))*X(J,2)*P(J,1,M)
   20 CONTINUE
      DO 30 J=0,JH
      P(J,2,M)=-M*X(J,1)*P(J,1,M)
   30 CONTINUE
 
      DO 40 N=M+1,MM
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 40 J=0,JH
      P(J,1,N)=E1*(N*X(J,1)*P(J,1,N-1)-P(J,2,N-1))
      P(J,2,N)=E2*P(J,1,N-1)-N*X(J,1)*P(J,1,N)
   40 CONTINUE

      RETURN
      END

