***********************************************************************
*     Dispersion Relation of Hough Functions Given Equivalent Depth.
*     wave number - frequency cross section
*       original   Ishioka,K.
*       last revised   Mar 1993   Horinouchi,T. 
*       (addapted to dcl-5, Apr 1997, Horiouchi)
***********************************************************************
c     parameters
c       MM: maximum total wave number for calculation 
c       HH: equivalent depth (=1.4D0*H)
c     variables
c       M : zonal wave number
c       E: eigen values which correspond nondimensional frequency 
c       A: matrix
*====================================================================== 
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(MM=80,KMM=3*(MM/2+1))
      PARAMETER(LM=KMM*(KMM+1)/2)
      DIMENSION A(LM),D(KMM),SD(KMM),E(KMM)
      REAL UX(LM,0:1),UY(LM,0:1)
 
      PI=4*ATAN(1D0)
      OME=2*PI/86400
*free osci      H=8D3  
*free osci      HH=1.4D0*H
      WRITE(6,*) 'EQIVALENT DEPTH [m] ?'
      READ(5,*) HH

      R=6.37D6
      G=9.8
      EPS=4*R*R*OME*OME/(G*HH)
 
      MMD=10
*     DE=1D-10
      DE=0
      I=1
 
      CALL INTUXY(LM,UX,UY)
 
      DO 20 M=0,MMD
      DO 10 IS=0,1
      CALL SETPAR(MM,M,IS,KM,NS,ID)
      CALL MATRIX(A,M,KM,NS,ID,EPS)
      CALL DTRID1(A,KM,D,SD,ICON)
      CALL DTRQL(D,SD,KM,E,KMD,ICON)
      CALL BOBBLE(KM,E)
      CALL CHANGE(KM,E)
      CALL SETUXY(M,KM,E,UX(I,IS),UY(I,IS),DE)
   10 CONTINUE
      I=I+KM
   20 CONTINUE
 
      CALL GROPEN
      CALL SLRAT(1.0,0.7)
      CALL GRAPH(LM,UX,UY,MMD,HH)
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE GRAPH(LM,UX,UY,MMD,HH)
 
      IMPLICIT REAL*8(A-H,O-Z)
      REAL UX(LM,0:1),UY(LM,0:1),UXMIN,UXMAX,UYMIN,UYMAX,DY1,DY2
      CHARACTER CSGI*1,CH*15,CS*6
 
      CH=' h=****.***(km)'
      WRITE(CH(4:11),'(F8.3)') HH*1D-3
**      CS='S (*O)'
**      CS(1:1)=CSGI(169)
**      CS(4:4)=CSGI(5)
**      CS(5:5)=CSGI(151)
      CS='(cpd)'
 
      UXMIN=-MMD
      UXMAX=MMD
      UYMIN=0
      IF(HH.GE.100) THEN
        UYMAX=2.
        DY1=0.2
        DY2=1.0
      ELSE
        UYMAX=.8
        DY1=0.1
        DY2=0.2
      ENDIF
 
      CALL GRFRM
      CALL SGPSET('LCLIP',.TRUE.)
      CALL GRSVPT(0.15,0.95,0.1,0.6)
      CALL GRSWND(UXMIN,UXMAX,UYMIN,UYMAX)
      CALL GRSTRN (1)
      CALL GRSTRF
      CALL UXAXDV('B',1.0,5.0)
      CALL UXAXDV('T',1.0,5.0)
      CALL UYAXDV('L',DY1,DY2)
      CALL UYAXDV('R',DY1,DY2)
      CALL UXSTTL('T',CH,0.0)
**      CALL UXSTTL('B','Wave Number s',0.0)
      CALL UXSTTL('B','Wave Number',0.0)
      CALL UYSTTL('L','Frequency '//CS,0.0)
      CALL SGSPMS(0.002)
      CALL SGSPMT(4)
      CALL SGPMU(LM,UX(1,0),UY(1,0))
      CALL SGSPMS(0.01)
      CALL SGSPMT(1)
      CALL SGPMU(LM,UX(1,1),UY(1,1))
 
      RETURN
      END
************************************************************************
      SUBROUTINE GROPEN
 
      CALL SGPWSN
      READ(5,*) IWS
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL GLPSET('LEPSL',.TRUE.)
      CALL GLPSET('LMISS',.TRUE.)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UZFACT(0.7)
      CALL UDPSET('LMSG',.FALSE.)
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETUXY(M,KM,E,UXD,UYD,DE)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(KM)
      REAL UXD(KM),UYD(KM)
 
      DO 10,K=1,KM
      ED=E(K)
      IF(ED.LE.-DE) THEN
      UXD(K)=-M
      UYD(K)=-ED
      ELSE IF(ED.GE.DE) THEN
      UXD(K)=M
      UYD(K)=ED
      END IF
   10 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETPAR(MM,M,IS,KM,NS,ID)
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      IE=MOD(MM-M,2)
      KM=3*((MM-M)/2+1)-(1-IE)*(1+IS)
      ID=IS
      NS=M
 
      IF(M.EQ.0) THEN
      KM=KM-(2-IS)
      ID=1-IS
      NS=M+1
      END IF
 
      RETURN
      END
************************************************************************
      SUBROUTINE INTUXY(LM,UX,UY)
 
      IMPLICIT REAL*8(A-H,O-Z)
      REAL UX(LM*2),UY(LM*2)
 
      DO 10 L=1,LM*2
      UX(L)=999.0
      UY(L)=999.0
   10 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE CHANGE(KM,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(KM)
 
      DO 10 K=1,KM
      E(K)=2*E(K)
   10 CONTINUE
 
      RETURN
      END
********************************************************************** *
      SUBROUTINE BOBBLE(KM,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(KM)
 
   20 CONTINUE
      INDEX=0
      DO 10 K=1,KM-1
      IF(E(K).GT.E(K+1)) THEN
      ED=E(K)
      E(K)=E(K+1)
      E(K+1)=ED
      INDEX=1
      END IF
   10 CONTINUE
      IF(INDEX.EQ.1) GOTO 20


      RETURN
      END
************************************************************************
      SUBROUTINE MATRIX(A,M,KM,NS,ID,EPS)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(KM*(KM+1)/2)
 
      R(N)=-1D0*M/(N*(N+1))
      S(N)=-SQRT(N*(N+1)/EPS)
      T(N)=SQRT(1D0*(N*N-1)*(N*N-M*M)/(4*N*N-1))/N
 
      DO 10 I=1,KM*(KM+1)/2
      A(I)=0
   10 CONTINUE
 
      N=NS
      L=1
      DO 20 K=1,KM
      JR=MOD(K-1-ID,3)
      IF(JR.NE.0) THEN
      A(L)=R(N)
      N=N+1
      END IF
      L=L+K+1
   20 CONTINUE
 
      N=NS+ID
      L=2
      DO 30 K=2,KM
      JS=MOD(K-2-ID,3)
      JT=MOD(K-ID,3)
      IF(JS.EQ.0) THEN
      A(L)=S(N)
      N=N+1
      ELSE IF(JT.EQ.0) THEN
      A(L)=T(N)
      N=N+1
      END IF
      L=L+K+1
   30 CONTINUE
 
      N=NS+2-ID
      L=4
      DO 40 K=3,KM
      JS=MOD(K-2-ID,3)
      IF(JS.EQ.0) THEN
      A(L)=T(N)
      N=N+2
      END IF
      L=L+K+1
   40 CONTINUE
 
      RETURN
      END
