c     Hough Functions for Given Frequency and Zonal Wave Number
c     (Eigen Function Problem for Eqivalent Deapth.)
c     Use expantion by Legenre assotiated function.
c     Plot geopotential field as function of latitude.
c     Warning : if there exists integer n0 s.t. M/SGM=-n0*(n0+1) and 
c    + n0 >= M , then USE ANOTHER PROGRAM .
c     Parameters :
c       MM:maximum of (total wave number)-(zonal wave number)
c    +     for calculation 
c       JH:meridional resolution to plot (grid number for hemisphere)
c       M :zonal wave number
c       SGM:normalized angular frequency s.t. SGM=1 for semidiurnal mode
c       NS:symmetric(=0) or antisymmetric(=1)
c       NN:dimesion of the matrix of coeficint
c       E:eigen values which correspond equivalent deapth.
c       EV:eigen vectors which correspond the coefficients of Legendre
c    +     associated function for geopotential field.
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      REAL*8 SGM
      PARAMETER(JH=120,MM=50,M=10,NS=0,NE=(MM+NS-M)/2,NN=NE-NS+1)
**      PARAMETER(SGM=-0.101D0)
      DIMENSION D(NN),SD(NN),E(NN),EV(NN,NN),EVD(NN)
      DIMENSION X(0:JH,2),PD(0:JH,2),P(0:JH,2,NN)
      DIMENSION UX(-JH:JH),UY(-JH:JH)
 
      WRITE(6,*) 'SGM?'
      READ(5,*) SGM
      CALL JUDGE(M,SGM)
      CALL MATRIX(M,SGM,NS,NN,D,SD)
      CALL DTEIG1(D,SD,NN,E,EV,NN,MD,ICON)
      CALL BUBBLE(NN,E,EV,EVD)
      CALL CHANGE(NN,E)
      CALL LEGEND(M,NS,NN,JH,X,PD,P)
      CALL GROPEN(-1)
      CALL SLMGN(0.0,0.0,0.0,0.05)
      CALL TITLE(M,NS,SGM)
      CALL SLDIV('T',2,4)
      CALL SLRAT(1.0,0.7)
**      DO 10 L=1,8
      DO 10 L=1,NN
      CALL GRAPH(NS,NN,JH,L,P,EV,UX,UY)
      CALL INFORM(NN,L,E)
   10 CONTINUE
**      DO 11 L=NN,NN-7,-1
**      CALL GRAPH(NS,NN,JH,L,P,EV,UX,UY)
**      CALL INFORM(NN,L,E)
**   11 CONTINUE
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE GROPEN(IWS)
 
      CALL GROPN(IWS)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UDPSET('LMSG',.FALSE.)
 
      RETURN
      END
************************************************************************
C     DIMENSIONALIZE EUIVALENT DEAPTHES

      SUBROUTINE CHANGE(NN,E)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN)
 
      A=6.37D6
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      G=9.8D0
      F=4*A*A*OME*OME/G*1D-3
 
      DO 20 ND=1,NN
      E(ND)=-F*E(ND)
   20 CONTINUE
 
      RETURN
      END
************************************************************************
C     SET THE COEFICINT MATRIX

      SUBROUTINE MATRIX(M,SGM,NS,NN,D,SD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION D(NN),SD(NN)
 
      R(N)=SQRT(1D0*(N*N-M*M)/(4*N*N-1))
      S(N)=M/SGM+N*(N+1)
      T1(N)=R(N)*R(N)*(N-1)*(N-1)/(S(N-1)*N*N)
      T2(N)=S(N)*SGM*SGM/(N*N*(N+1)*(N+1))
      T3(N)=R(N+1)*R(N+1)*(N+2)*(N+2)/(S(N+1)*(N+1)*(N+1))
 
      DO 10 ND=2,NN
        N=M+NS+ND*2-2
        D(ND)=T1(N)-T2(N)+T3(N)
        N=N-1
        SD(ND)=R(N)*R(N+1)/S(N)
   10 CONTINUE
 
      IF(NS.EQ.0) THEN
        N=M
        D(1)=-T2(N)+T3(N)
      ELSE
        N=M+1
        D(1)=T1(N)-T2(N)+T3(N)
      END IF
      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(NS,NN,JH,L,P,EV,UX,UY)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION EV(NN,NN)
      DIMENSION P(0:JH,2,NN)
      DIMENSION UX(-JH:JH),UY(-JH:JH)
 
      DO 10 J=-JH,JH
      UX(J)=90.0*J/JH
      UY(J)=0
   10 CONTINUE
 
      DO 20 ND=1,NN
      DO 20 J=0,JH
      UY(J)=UY(J)+EV(ND,L)*P(J,1,ND)
   20 CONTINUE
 
      IF(NS.EQ.0) THEN
      DO 30 J=0,JH
      UY(-J)=UY(J)
   30 CONTINUE
      ELSE
      DO 40 J=0,JH
      UY(-J)=-UY(J)
   40 CONTINUE
      END IF
 
      UYMAX=1E-10
      DO 50 J=0,JH
      UYA=ABS(UY(J))
      IF(UYA.GT.UYMAX) THEN
      UYMAX=UYA
      F=UY(J)/UYA
      END IF
   50 CONTINUE
 
      IF(F.LT.0) THEN
      DO 60 J=-JH,JH
      UY(J)=-UY(J)
   60 CONTINUE
      END IF
 
      CALL GRFRM
      CALL GRSVPT(0.1,0.9,0.1,0.6)
      UX1=-90
      UX2=90
      UY1=-UYMAX*1.1
      UY2=UYMAX*1.1
      CALL GRSWND(UX1,UX2,UY1,UY2)
      CALL GRSTRN (1)
      CALL GRSTRF
      CALL UXAXDV('B',10.0,30.0)
      CALL UXAXDV('T',10.0,30.0)
      CALL UYPAXS('L',1)
      CALL UYPAXS('R',1)
      CALL UXPAXS('U',1)
      CALL UYPAXS('U',1)
      CALL SGPLU(JH*2+1,UX,UY)
 
      RETURN
      END
************************************************************************
      SUBROUTINE INFORM(NN,L,E)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      DIMENSION E(NN)
      CHARACTER CH*15
 
      CH(1:2)='H='
      CH(12:15)='(KM)'
      WRITE(CH(3:11),'(F9.5)') E(L)
 
      CALL UXMTTL('T',CH,0.0)
 
      RETURN
      END
************************************************************************
      SUBROUTINE TITLE(M,NS,SGM)
 
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER CS*5,CT*13,CU*14,CSGI*1
 
      CS='S=## '
      WRITE(CS(3:4),'(I2)') M
      CU(1:2)=CSGI(169)//'='
      CU(11:14)='('//CSGI(5)//CSGI(151)//')'
      WRITE(CU(3:10),'(F8.4)') SGM*2
 
      CT='  (SYMMETRIC)'
      IF(NS.EQ.1) CT=' (ANTISYMMETRIC)'
 
      CALL SLSTTL('HOUGH FUNCTIONS '//CS//CU//CT,'T',0.0,0.0,0.015,1)
 
      RETURN
      END
************************************************************************
c     set Legendre polynominals.

      SUBROUTINE LEGEND(M,NS,NN,JH,X,PD,P)
c     M : zonal wave number
c     X : cos , sin
c     p(*,*,n) : Legendre zonal wave num. M ,total 2*(N-1)+NS+M
c     p(*,1,*) : P(x)
c     p(*,2,*) : (1-x**2)(d/dx)P(x)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(0:JH,2),PD(0:JH,2),P(0:JH,2,NN)
 
      PAI=4*ATAN(1D0)
 
      DO 10 J=0,JH
      X(J,1)=SIN(PAI/2*J/JH)
      X(J,2)=COS(PAI/2*J/JH)
      PD(J,1)=1
   10 CONTINUE
 
      DO 20 MD=1,M
      DO 20 J=0,JH
      PD(J,1)=SQRT(1D0*(2*MD+1)/(2*MD))*X(J,2)*PD(J,1)
   20 CONTINUE
      DO 21 J=0,JH
      PD(J,2)=-M*X(J,1)*PD(J,1)
   21 CONTINUE
 
      IF(NS.EQ.0) THEN
      DO 30 J=0,JH
      P(J,1,1)=PD(J,1)
      P(J,2,1)=PD(J,2)
   30 CONTINUE
      ELSE
      N=M+1
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 31 J=0,JH
      P(J,1,1)=E1*(N*X(J,1)*PD(J,1)-PD(J,2))
      P(J,2,1)=E2*PD(J,1)-N*X(J,1)*P(J,1,1)
   31 CONTINUE
      END IF
 
      DO 41 ND=2,NN
      N=M+NS+ND*2-3
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 40 J=0,JH
      PD(J,1)=E1*(N*X(J,1)*P(J,1,ND-1)-P(J,2,ND-1))
      PD(J,2)=E2*P(J,1,ND-1)-N*X(J,1)*PD(J,1)
   40 CONTINUE
      N=N+1
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 41 J=0,JH
      P(J,1,ND)=E1*(N*X(J,1)*PD(J,1)-PD(J,2))
      P(J,2,ND)=E2*PD(J,1)-N*X(J,1)*P(J,1,ND)
   41 CONTINUE
 
      RETURN
      END
******************************************************************
      SUBROUTINE JUDGE(M,SGM)

      IMPLICIT REAL*8(A-H,O-Z)
      IF (SGM.EQ.0) THEN
        WRITE(6,*) 'SGM=0'
        STOP
      ENDIF
      IF (M/SGM.GE.0) RETURN
      S=(-1D0+SQRT(1D0-4*M/SGM))/2
      SS=INT(S)
      IF (S.EQ.SS .AND. SS.GE.M) THEN
        WRITE(6,*) 'THERE EXISTS INTEGER N S.T. M/SGM=-N*(N+1)'
        STOP
      ENDIF
      RETURN
      END
********************************************************************** *
c     sorting by 'awadachi' method

      SUBROUTINE BUBBLE(NN,E,EV,EVD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(NN),EV(NN,NN),EVD(NN)
 
   30 CONTINUE
      INDEX=0
      DO 20 N=1,NN-1
      IF(E(N).GT.E(N+1)) THEN
      ED=E(N)
      E(N)=E(N+1)
      E(N+1)=ED
      DO 10 L=1,NN
      EVD(L)=EV(L,N)
      EV(L,N)=EV(L,N+1)
      EV(L,N+1)=EVD(L)
   10 CONTINUE
      INDEX=1      
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30
 
      RETURN
      END
