********************************************************************************
* ɽ ϵήϳ ׻¸
*
*      Holton and Lindzen model for equatorial Quasi-Biennial Oscillation
*      Slide Show of U, DU1, DU2, F1, F2
*
*  97/11/19  ľ
*      99/06/26 ; 
********************************************************************************
      program flux

      include 'incpa.f'
      real d(0:lev),u(0:lev),f1(0:lev),f2(0:lev),du1(0:lev),du2(0:lev)
*-----+--1---------2---------3---------4---------5---------6---------7-+
      open(51, file='qbo1.data', form='unformatted')
      read(51)    a1, a2, rc1, rc2, rk1, rk2, d, e
      call gr2ini(a1, a2, rc1, rc2, rk1, rk2)       ! graphics initialization
      call grduma(dumax)
      call gr2in2(dumax)

      do i=1,imax
        read(51)    t, u, f1, f2, du1, du2
        call graph2(t, u, f1, f2, du1, du2)
      end do

      close(51)
      call grcls
      stop
      end
********************************************************************************
      subroutine gr2ini(ra1, ra2, rrc1, rrc2, rrk1, rrk2)

      include 'incpa.f'
      parameter(vymi=0.1,vyma=0.64)

      real u(0:lev),f1(0:lev),f2(0:lev),du1(0:lev),du2(0:lev)
      real rlev(0:lev+2),fl(0:lev+2)
      character cttl*60,ctime*5

      save
*-----+--1---------2---------3---------4---------5---------6---------7-+
       a1 =  ra1
       a2 =  ra2
      rc1 = rrc1
      rc2 = rrc2
      rk1 = rrk1
      rk2 = rrk2

      zmax = lev*dz
 
      do j=0,lev
        rlev(j) = j*dz
      end do
      rlev(lev+1) = rlev(lev)
      rlev(lev+2) = 0.0

      call glrget('rundef', rundef)
      call swiset('iwidth', 720)
      call swiset('iheight',540)
      call swlset('lwait',  .false.)
      call swlset('lalt',   .true.)
      call sglset('lcorner',.false.)
      call sglset('lcntl',  .true.)
      call sglset('lclip',  .true.)
      call sglset('lfull',  .true.)
      call sglset('lprop', .false.)
      call uzfact(0.7)
      call uzrget('rsizec1', rsizec1)
      call gropn(1)

      return
*===============================================================================
      entry gr2in2(rdumax)

      dumax = rdumax

      return
*===============================================================================
      entry graph2(t, u, f1, f2, du1, du2)

      flow = max(abs(rc1), abs(rc2))
      flux = max(abs(f1(0)), abs(f2(0)))

* F2
      call grfrm
      call grswnd(-flux,  0.0,  0.0, zmax)
      call grsvpt( 0.10, 0.23, vymi, vyma)
      call grstrn(1)
      call grstrf

      do j=0,lev
        fl(j) = f2(j)
      end do
      fl(lev+1) = 0.0
      fl(lev+2) = 0.0

      call sgstnp(9999)
      call sgtnu(lev+3, fl, rlev)
      call sgspli(43)
      call sgplu(lev+1, f2, rlev)

      call uxaxdv('b', flux/5, flux)
      call uxaxdv('t', flux/5, flux)
      call uzlset('labelyl',.true.)
      call uyaxdv('l', 0.2, 1.0)
      call uzlset('labelyl',.false.)
      call uyaxdv('r', 0.2, 1.0) 
      call uxsttl('b', 'F_2"', 0.0)
      call uysttl('l', 'HEIGHT', 0.0)

      cttl = 'a_2"=??.?? c_2"=?.?? k_2"=??.? '
      write(cttl( 6:10),'(f5.2)')  a2
      write(cttl(17:20),'(f4.1)') rc2
      write(cttl(27:30),'(f4.1)') rk2
      call uxsttl('t', cttl, -1.0)
      cttl = 'a_1"=??.?? c_1"=??.? k_1"=??.? '
      write(cttl( 6:10),'(f5.2)')  a1
      write(cttl(17:20),'(f4.1)') rc1
      write(cttl(27:30),'(f4.1)') rk1
      call uxsttl('t', cttl, -1.0)

* (dU/dt)_2
      call grfig
      call grswnd(-dumax,  0.0,  0.0, zmax)
      call grsvpt(  0.24, 0.37, vymi, vyma)
      call grstrn(1)
      call grstrf

      call sgrset('const', 0.01)
      call sgslai(91)
      do j=0,lev-1
        za = j*dz
        if(abs(du2(j)) .gt. 0.04*dumax) then
          call sgrset('const', 0.01)
        else
          call sgrset('const', 0.0)
        end if
        if(abs(du2(j)) .gt. 1.e-3*dumax) call sglau(0., za, du2(j), za)
      end do

      call sgspli(43)
      call sgplu(lev+1, du2, rlev)

      call uzlset('labelxb',.false.)
      call uxaxdv('b', dumax/5, dumax)
      call uzlset('labelxb',.true.)
      call uxaxdv('t', dumax/5, dumax)
      call uyaxdv('l', 0.2, 1.0)
      call uyaxdv('r', 0.2, 1.0) 
      call uzrset('rsizec1', 0.7*rsizec1)
      call uxsttl('b', ' ', 0.0)
      call uzrset('rsizec1', rsizec1)
      call uxsttl('b', '(dU/dt)_2"', 0.0)

* U
      xmin = rc2*0.9
      xmax = rc1*0.9
      call grfig
      call grsvpt(0.38, 0.68, vymi, vyma)
      call grswnd(xmin, xmax, 0.0, zmax)
      call grstrn(1)
      call grstrf

      call sgslai(671)
      do j=0,lev-1
        za = j*dz
        if(abs(u(j)) .gt. 0.04*flow) then
          call sgrset('const', 0.01)
        else
          call sgrset('const', 0.0)
        end if
        if(abs(u(j)) .gt. 1.e-3*flow) call sglau(0., za, u(j), za)
      end do

      call sgrset('const', 0.0)
      call sgslai(1)
      call sglau(0.0, 0.0,  0.0, zmax)

      call sgspli(33)
      call sgplu(lev+1, u, rlev)

      call uxaxdv('b', flow/10, flow/2)
      call uxaxdv('t', flow/10, flow/2)
      call uyaxdv('l', 0.2, 1.0)
      call uyaxdv('r', 0.2, 1.0) 
      call uxsttl('b', 'U', 0.0)

* (dU/dt)_1 
      call grfig
      call grswnd( 0.0, dumax,  0.0, zmax)
      call grsvpt(0.69,  0.82, vymi, vyma)
      call grstrn(1)
      call grstrf

      call sgslai(81)
      do j=0,lev-1
        za = j*dz
        if(abs(du1(j)) .gt. 0.04*dumax) then
          call sgrset('const', 0.01)
        else
          call sgrset('const', 0.0)
        end if  
        if(abs(du1(j)) .gt. 1.e-3*dumax) call sglau(0., za, du1(j), za)
      end do

      call sgspli(23)
      call sgplu(lev+1, du1, rlev)

      call uzlset('labelxb',.false.)
      call uxaxdv('b', dumax/5, dumax)
      call uzlset('labelxb',.true.)
      call uxaxdv('t', dumax/5, dumax)
      call uyaxdv('l', 0.2, 1.0)
      call uyaxdv('r', 0.2, 1.0) 
      call uzrset('rsizec1', 0.7*rsizec1)
      call uxsttl('b', ' ', 0.0)
      call uzrset('rsizec1', rsizec1)
      call uxsttl('b', '(dU/dt)_1"', 0.0)

* F1
      call grfig
      call grswnd( 0.0, flux,  0.0, zmax)
      call grsvpt(0.83, 0.96, vymi, vyma)
      call grstrn(1)
      call grstrf

      do j=0,lev
        fl(j) = f1(j)
      end do
      fl(lev+1) = 0.0
      fl(lev+2) = 0.0

      call sgstnp(8999)
      call sgtnu(lev+3, fl, rlev)
      call sgspli(23)
      call sgplu(lev+1, f1, rlev)

      call uxaxdv('b', flux/5, flux)
      call uxaxdv('t', flux/5, flux)
      call uyaxdv('l', 0.2, 1.0)
      call uyaxdv('r', 0.2, 1.0) 
      call uxsttl('b', 'F_1"', 0.0)

      write(ctime,'(f5.1)') t
      call uxmttl('t', ' ', 1.0)
      call uxmttl('t', 'T='//ctime, 1.0)

      return
      end
********************************************************************************
      subroutine grduma(dumax)

      include 'incpa.f'
      real u(0:lev),f1(0:lev),f2(0:lev),du1(0:lev),du2(0:lev)
*-----+--1---------2---------3---------4---------5---------6---------7-+
      du1max = -999.
      du2max = -999.
      do i=1,imax
        read(51)    t, u, f1, f2, du1, du2
        du1ma = rmax(du1, lev+1, 1)
        if(abs(du1ma) .gt. du1max) du1max = abs(du1ma)
        du1mi = rmin(du1, lev+1, 1)
        if(abs(du1mi) .gt. du1max) du1max = abs(du1mi)
        du2ma = rmax(du2, lev+1, 1)
        if(abs(du2ma) .gt. du2max) du2max = abs(du2ma)
        du2mi = rmin(du2, lev+1, 1)
        if(abs(du2mi) .gt. du2max) du2max = abs(du2mi)
      end do
      dumax = max(du1max, du2max)
      dumax = rgnge(dumax) /2.

      close(51)
      open (51, file='qbo1.data', form='unformatted')
      read (51)    a1, a2, rc1, rc2, rk1, rk2, d, e

      return
      end
********************************************************************************
