********************************************************************************
* ɽ ϵήϳ ׻¸
*
*      Holton and Lindzen model for equatorial Quasi-Biennial Oscillation
*      < Plumb version >
*
*  87/01/07 ; (Jan. 7, 1987; Aug.31, 1990; Feb.14, 1996)
*      90/10/18  
*      98/05/06  ľ
*      99/06/26 ; 
********************************************************************************
      program qbo
 
      include 'incpa.f'
      real u(0:lev),du(0:lev),work(0:lev,3)
      real d(0:lev)
      character cdummy*60

      common/consts/a1,a2,rc1,rc2,rk1,rk2,d,e,dzdz
      external odrkg,fcn
*-----+--1---------2---------3---------4---------5---------6---------7-+
*-- read experimental parameters
      open (51,file='expar.data')
      read (51,*) a1, a2, rc1, rc2, rk1, rk2, cdummy
      close(51)

      call qboini                                   ! set constants
      call gr1ini(a1, a2, rc1, rc2, rk1, rk2)       ! graphics initialization
      call init(u, lev, dz)                         ! set initial conditions

*-- time integration
      t = 0.0
      do i=1,imax
        call odrkdu(lev+1, odrkg, fcn, t, t+dt, istep, u, work)
      end do
*-- restart
      t = 0.0
      do i=1,imax
        call odrkdu(lev+1, odrkg, fcn, t, t+dt, istep, u, work)
        call graph1(i, u)
        call fcn(n, t, u, du)
        call outdat(t, u)
      end do

      call qbocls
      call grcls
      stop
      end
********************************************************************************
      subroutine init(u, lev, dz)

      real u(0:lev)

      u0 = -0.1
      do j=0,lev
        u(j) = - 0.03*j*dz
        if(u(j) .lt. u0) u(j) = u0
      end do

      return
      end
********************************************************************************
      subroutine fcn(n, t, u, du)

      include 'incpa.f'
      real  d(0:lev),  u(0:lev), du(0:lev), du1(0:lev), du2(0:lev)
      real f1(0:lev), b1(0:lev), g1(0:lev)
      real f2(0:lev), b2(0:lev), g2(0:lev)

      common/consts/a1,a2,rc1,rc2,rk1,rk2,d,e,dzdz
      save
*-----+--1---------2---------3---------4---------5---------6---------7-+
*-- boundary condition
      du(  0) = 0.0
      du(lev) = 0.0
      u (lev) = u(lev-1)

*-- du/dt
      do j=0,lev
        g1(j) = d(j)/(rk1*(u(j)-rc1)**2)
        g2(j) = d(j)/(rk2*(u(j)-rc2)**2)
      end do

      b1(0) = 0.0
      b2(0) = 0.0
      f1(0) = a1*exp(b1(0))
      f2(0) = a2*exp(b2(0))
      do j=1,lev
        b1(j) = b1(j-1) - 0.5*(g1(j-1)+g1(j))*dz
        b2(j) = b2(j-1) - 0.5*(g2(j-1)+g2(j))*dz
        f1(j) = a1*exp(b1(j))
        f2(j) = a2*exp(b2(j))
        du1(j) = f1(j)*g1(j)
        du2(j) = f2(j)*g2(j)
      end do

      do j=1,lev-1
        du(j) = du1(j) + du2(j) + e*(u(j+1) - 2.0*u(j) + u(j-1))/dzdz
      end do

      return
*===============================================================================
      entry qboini

      dzdz = dz*dz

      do j=0,lev
        d(j) = 1.0
      end do
      e = 0.02

      open (61, file='qbo1.data', form='unformatted')
      write(61) a1, a2, rc1, rc2, rk1, rk2, d, e

      return
*===============================================================================
      entry outdat(t, u)

      write(61) t, u, f1, f2, du1, du2

      return
*===============================================================================
      entry qbocls

      close(61)

      return
      end
********************************************************************************
      subroutine gr1ini(a1, a2, rc1, rc2, rk1, rk2)

      include 'incpa.f'
      parameter(ntone=69,ipat0=21,idlev=1) ! , ֹν, ʬ
      parameter(vxmi=0.15,vxma=0.95,vymi=0.14,vyma=0.64)

      real u(0:lev),rbar(2*ntone,2),ubar(2,0:lev)
      character cttl*60

      save
*-----+--1---------2---------3---------4---------5---------6---------7-+
      call gllset('lmiss',  .true.)
      call glrget('rmiss',  rmiss)
      call swiset('iwidth', 720)
      call swiset('iheight',540)
      call sglset('lcorner',.false.)
      call sglset('lcntl',  .true.)
      call sglset('lfull',  .true.)
      call uzfact(0.8)

      call uzrget('rsizel1', rsizel1)

* uepack 
      rbmin = min(rc1, rc2)
      rbmax = max(rc1, rc2)
      dlev = (rbmax-rbmin)/ntone
      do i=1,ntone
        ipat = (ipat0 + (i-1)*idlev)*1000 + 999
        tlev1 = rbmin + (i-1)*dlev
        tlev2 = tlev1 + dlev
        if(i .eq.     1) tlev1 = rmiss
        if(i .eq. ntone) tlev2 = rmiss
        call uestlv(tlev1, tlev2, ipat)
      end do
      call uestlv(-0.1*dlev, 0.1*dlev, 99999)

      do i=1,2*ntone
        rbar(i,1) = rbmin + (rbmax-rbmin)*(i-1)/(2*ntone-1)
        rbar(i,2) = rbar(i,1)
      end do
      dxma = (rbmax-rbmin)/2
      dxma = rgnle(dxma)
      dxmi = dxma/2

      call gropn(1)
      call grfrm

* color bar
      call grswnd(rbmin, rbmax,  0.0,  1.0)
      call grsvpt( 0.15,  0.35, 0.03, 0.06)
      call grstrn(1)
      call grstrf

      call uetone(rbar, 2*ntone, 2*ntone, 2)
      call uzrset('rsizel1', 0.013)
      call uxaxdv('b', dxmi, dxma) 
      call uzrset('rsizel1', rsizel1)
      call uxaxdv('t', dxmi, dxma) 
      call uzlset('labelyl',.false.)
      call uyaxdv('l', 1., 1.)
      call uzlset('labelyl',.true.)
      call uyaxdv('r', 1., 1.) 
      call uxsttl('t', 'U', 0.0)

* main frame
      call grfig
      tmax = imax*dt
      zmax =  lev*dz
      call grswnd( 0.0, tmax,  0.0, zmax)
      call grsvpt(vxmi, vxma, vymi, vyma)
      call grstrn(1)
      call grstrf

      call uziset('inner', -1)
      call uxaxdv('b', 20., 100.)
      call uxaxdv('t', 20., 100.) 
      call uyaxdv('l', 0.2, 1.0)
      call uyaxdv('r', 0.2, 1.0) 
      call uxsttl('b', 'TIME', 0.0)
      call uysttl('l', 'HEIGHT', 0.0)

      cttl = 'a_2"=??.?? c_2"=?.?? k_2"=??.? '
      write(cttl( 6:10),'(f5.2)')  a2
      write(cttl(17:20),'(f4.1)') rc2
      write(cttl(27:30),'(f4.1)') rk2
      call uxsttl('t', cttl, -1.0)
      cttl = 'a_1"=??.?? c_1"=??.? k_1"=??.? '
      write(cttl( 6:10),'(f5.2)')  a1
      write(cttl(17:20),'(f4.1)') rc1
      write(cttl(27:30),'(f4.1)') rk1
      call uxsttl('t', cttl, -1.0)

* setup for ubar
      vxdel = (vxma-vxmi)/imax
      vydel = (vyma-vymi)/lev

      return
*===============================================================================
      entry graph1(ij, u)

      call grfig
      vvxmi =  vxmi + (ij-1)*vxdel  
      vvxma = vvxmi +  vxdel
      call grswnd( 0.0,    1.0,  0.0,  1.0)
      call grsvpt(vvxmi, vvxma, vymi, vyma)
      call grstrn(1)
      call grstrf

      do l=0,lev
        ubar(1,l) = u(l)
        ubar(2,l) = u(l)
      end do

      call uetone(ubar, 2, 2, lev+1)

      return
      end
********************************************************************************

